/*
	$Id: HardwareTransfer.java 4512 2008-09-11 20:05:50Z jari $

	Copyright (C) 2005 Jari Hakkinen, Gregory Vincic
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg, Gregory Vincic
	Copyright (C) 2007 Jari Hakkinen, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

package net.sf.basedb.clients.migrate;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.CommonItem;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Hardware;
import net.sf.basedb.core.HardwareType;
import net.sf.basedb.core.SystemItems;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
<dl>
<dt><b>Base 1 tables</b></dt>
<dd>Hardware</dd>
<dt><b>Unused columns</b></dt>
<dd>addedDate</dd>
<dt><b>Notes</b></dt>
<dd>
<ul>
<li>Base 1 only has hardware of {@link HardwareType#SCANNER}</li>
</ul>
</dd>
</dl>

@author Gregory
@version 2.0
@base.modified $Date: 2008-09-11 22:05:50 +0200 (Thu, 11 Sep 2008) $
*/

public class HardwareTransfer extends CommonItemTransfer
{
	/**
	Id for the HardwareType.SCANNER
	 */
	private int scannerId = 0;
	
	/**
	Logger for this class
	 */
	private static final org.apache.log4j.Logger log = org.apache.log4j.Logger.getLogger("net.sf.basedb.clients.migrate.HardwareTransfer");

	/**
	Creates a new HardwareTransfer.
	
	@param userT <code>UserTransfer</code>
	@param groupT <code>GroupTransfer</code>
	 */
	HardwareTransfer(UserTransfer userT, GroupTransfer groupT)
	{
		super(userT, groupT);
		scannerId = SystemItems.getId(HardwareType.SCANNER);
	}

	public void start()
	{
		log.info("Transferring Hardware...");
		int count = getInt("countHardware");
		initIdMap(count);
		startProgress(count, "Hardwares");
		run(prepareStatementFromFile("selectHardware"));
		done();
		log.info("Transferring Hardware...DONE");
	}

	/**
	Returns a Hardware from the current row in a given ResultSet.
	
	@param b2c {@link DbControl}
	@param rs {@link ResultSet}
	@return Hardware
	 */
	protected Hardware createItem(DbControl b2c, ResultSet rs)
			throws BaseException, SQLException
	{
		Hardware hardware = HardwareType.getById(b2c, scannerId).newHardware();
		setCommonItemData(b2c, (CommonItem) hardware, rs);
		hardware.setVersionString(rs.getString(9));
		if (log.isDebugEnabled())
		{
			log.debug("Transfered Hardware: " + hardware.getName());
			log.debug("--------------------------------------------------------------------------");
		}
		return hardware;
	}

	/**
	Returns a Hardware by the given base 1 hardware id.
	
	@param dbc <code>DbControl</code> used to get the hardware.
	@param base1HardwareId
	@return <code>Hardware</code> or null if the hardware doesn't exist;
	 */
	public Hardware getHardwareById(DbControl dbc, int base1HardwareId)
	{
		try
		{
			Integer id = getBase2Id(base1HardwareId);
			return id == null ? null : Hardware.getById(dbc, id);
		}
		catch (BaseException e)
		{
			log.warn("Couldn't get hardware in BASE1 by id: " + base1HardwareId);
			if(log.isDebugEnabled()) log.debug("\t" + e);
		}
		return null;
	}
}
