/*
	$Id: Scripts.java 4512 2008-09-11 20:05:50Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.taglib.extensions;

import java.util.Collection;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.tagext.TagSupport;

import net.sf.basedb.clients.web.extensions.JspContext;

/**
	Tag for creating &lt;script&gt; tags from a {@link JspContext}.

	<p>
	<b>Syntax:</b>
	<pre>
   &lt;ext:scripts 
      context=...
  /&gt;
</pre>
	<p>
	<table border=1 cellspacing=0 cellpadding=2>
	<tr>
		<td><b>Attribute</b></td>
		<td><b>Default value</b></td>
		<td><b>Required</b></td>
		<td><b>Description</b></td>
	</tr>
	<tr>
		<td>context</td>
		<td>-</td>
		<td>yes</td>
		<td>
			A {@link JspContext} to pick scripts from.
		</td>
	</tr>
	</table>

	@author Nicklas
	@version 2.7
*/
public class Scripts 
	extends TagSupport
{

	private static final long serialVersionUID = -2867817254135934000L;

	// The JSP context
	private JspContext context = null;

	public void setContext(JspContext context)
	{
		this.context = context;
	}

	public int doStartTag() 
		throws JspException
	{
		if (context == null) return SKIP_BODY;

		Collection<String> scripts = context.getScripts();
		if (scripts == null) return SKIP_BODY;
		
		StringBuilder sb =new StringBuilder();
		for (String script : scripts)
		{
			sb.append("<script language=\"JavaScript\" type=\"text/javascript\" src=\"");
			sb.append(script);
			sb.append("\"></script>\n");
		}
		
		try
		{
			pageContext.getOut().print(sb.toString());
    	}
		catch (Exception ex)
		{
			throw new JspTagException(ex.getMessage());
		}
		return SKIP_BODY;
	}
}


