/*
	$Id: Panel.java 4512 2008-09-11 20:05:50Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.taglib.table;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.tagext.TagSupport;

/**
	This tag creates a panel in the table. The tag must be located inside
	a {@link Table &lt;tbl:table&gt;}
	tag.

	<p>
	<b>Syntax:</b>
	<pre>
   &lt;tbl:panel
      id=...
      clazz=...
      style=...
      visible=true|false
   &gt;
</pre>
	<p>
	<table border=1 cellspacing=0 cellpadding=2>
	<tr>
		<td><b>Attribute</b></td>
		<td><b>Default value</b></td>
		<td><b>Required</b></td>
		<td><b>Description</b></td>
	</tr>
	<tr>
		<td>id</td>
		<td>-</td>
		<td>no</td>
		<td>
			The ID of the panel. The value of this attribute
			goes directly into the standard HTML <code>id</code> attribute.
			This means that if you need a reference to the table object in your own
			JavaScript code, you may for example use the following code:
			<pre class="code">
toolbar = document.getElementById('&lt;id&gt;');
</pre>
		</td>
	</tr>
	<tr>
		<td>clazz</td>
		<td>panel</td>
		<td>no</td>
		<td>
			The value if this attribute goes directly into the standard HTML 
			<code>class</code> attribute. This allows you to apply different styles
			to different tables.
		</td>
	</tr>
	<tr>
		<td>style</td>
		<td>-</td>
		<td>no</td>
		<td>
			The value if this attribute goes directly into the standard HTML 
			<code>style</code> attribute. This allows you to apply different styles
			to different tables.
		</td>
	</tr>
	<tr>
		<td>visible</td>
		<td>true</td>
		<td>no</td>
		<td>
			If the panel should be visible or not.
		</td>
	</tr>
	</table>
	<p>

	The <code>&lt;tbl:panel&gt;</code> doesn't have any subtags.

	@author Nicklas
	@version 2.0
*/
public class Panel
	extends TagSupport
{

	/**
		
	*/
	private static final long serialVersionUID = 8435173953529878997L;

	/**
		The parent &lt;tbl:table&gt; tag.
	*/
	private Table table;

	/**
		Optional <code>class</code> attribute
	*/
	private String clazz = "panel";
	
	/**
		Optional <code>style</code> attribute
	*/
	private String style = null;

	/**
		If the panel should be visible or not.
	*/
	private boolean visible = true;

	/*
		Taglib initialization methods
	*/
	public void setClazz(String clazz)
	{
		this.clazz = clazz;
	}
	public String getClazz()
	{
		return clazz;
	}

	public void setStyle(String style)
	{
		this.style = style;
	}
	public String getStyle()
	{
		return style;
	}

	public void setVisible(boolean visible)
	{
		this.visible = visible;
	}
	public boolean isVisible()
	{
		return visible;
	}
	/*
		Package methods
	*/
	Table getTable()
	{
		return table;
	}

	/*
		Display methods
	*/
	public int doStartTag()
		throws JspException
	{
		//if (!(getParent() instanceof Table)) throw new JspTagException("Tag <tbl:panel> must be inside a <tbl:table> tag");
		//table = (Table)getParent();
		if (!isVisible()) return SKIP_BODY;

		StringBuilder sb = new StringBuilder();
		sb.append("<tr><td");
		if (getId() != null) sb.append(" id=\"").append(getId()).append("\"");
		if (getClazz() != null) sb.append(" class=\"").append(getClazz()).append("\"");
		if (getStyle() != null) sb.append(" style=\"").append(getStyle()).append("\"");
		sb.append(">\n");
		try
		{
			pageContext.getOut().print(sb.toString());
		}
		catch (Exception ex)
		{
			throw new JspTagException(ex.getMessage());
		}
		return EVAL_BODY_INCLUDE;
	}

	public int doEndTag()
		throws JspException
	{
		if (!isVisible()) return EVAL_PAGE;
		StringBuilder sb = new StringBuilder();
		sb.append("</td></tr>\n");
		try
		{
			pageContext.getOut().print(sb.toString());
		}
		catch (Exception ex)
		{
			throw new JspTagException(ex.getMessage());
		}
		return EVAL_PAGE;
	}


}

