/*
	$Id: CommonItem.java 4517 2008-09-11 20:11:02Z jari $

	Copyright (C) 2005 Samuel Andersson, Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

package net.sf.basedb.core;

import net.sf.basedb.core.data.CommonData;
import net.sf.basedb.info.CommonItemInfo;

/**
	@author Nicklas
	@version 2.0
*/
public abstract class CommonItem<D extends CommonData>
	extends SharedItem<D>
	implements Nameable, Removable
{
	CommonItem(D commonData)
	{
		super(commonData);
	}

	// -------------------------------------------
	/*
		From the Nameable interface
		-------------------------------------------
	*/
	public String getName()
	{
		return getData().getName();
	}
	public void setName(String name)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		NameableUtil.setName(getData(), name);
	}
	public String getDescription()
	{
		return getData().getDescription();
	}
	public void setDescription(String description)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		NameableUtil.setDescription(getData(), description);
	}
	// -------------------------------------------
	/*
		From the Removable interface
		-------------------------------------------
	*/
	public boolean isRemoved()
	{
		return getData().isRemoved();
	}
	public void setRemoved(boolean removed)
		throws PermissionDeniedException
	{
		checkPermission(removed ? Permission.DELETE : Permission.WRITE);
		getData().setRemoved(removed);
	}
	
	/**
	 	@since 2.5
	 */
	CommonItemInfo toTransferable(CommonItemInfo info)
	{
		super.toTransferable(info);
		info.setDescription(this.getDescription());
		info.setName(this.getName());
		return info;
	}
}



