/*
	$Id: ContextKey.java 4517 2008-09-11 20:11:02Z jari $

	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

/**

	@author Nicklas
	@version 2.0
	@see <a href="../../../../../html/developerdoc/api/data_api.clients.html">Session and client overview</a>
	@base.modified $Date: 2008-09-11 22:11:02 +0200 (Thu, 11 Sep 2008) $
*/
class ContextKey
{

	private final Item item;
	private final String subContext;
	
	/**
	 	Create a new ContextKey object from {@link Item} and it's subcontext.
	  	@param item
	  	@param subContext
	*/
	public ContextKey(Item item, String subContext)
	{
		this.item = item;
		this.subContext = subContext;
	}

	/**	  	
	  	@return The <code>Item</code> that belongs to this object.
	*/
	public Item getItem()
	{
		return item;
	}
	
	/**	
	  	@return A <code>String</code> with the subcontext.
	*/
	public String getSubContext()
	{
		return subContext;
	}

	/**
		Check if this object is equal to another <code>ContextKey</code>
		object. They are equal if both have the same item and subcontext.
	*/
	public boolean equals(Object o)
	{
		if ((o == null) || (getClass() != o.getClass()))
		{
			return false;
		}
		ContextKey ck = (ContextKey)o;
		return (this.item == ck.item) && this.subContext.equals(ck.subContext);
	}

	/**
		Calculate the hash code for the object.
	*/
	public int hashCode()
	{
		return (int)53 * item.getValue() + 17 * subContext.hashCode();
	}
}
