/*
	$Id$

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg, Martin Svensson
	
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.data.FeatureData;
import net.sf.basedb.core.data.ArrayDesignBlockData;
import net.sf.basedb.core.data.WellData;

import org.hibernate.EntityMode;
import org.hibernate.metadata.ClassMetadata;

/**
	This is a utility class for Features.
	
	@base.modified $Date$
	@author Enell, Nicklas
	@version 2.0
*/
public class Feature
{
	
	private static ClassMetadata metaData = HibernateUtil.getClassMetadata(FeatureData.class.getName());

	/**
		Get the array design block from a feature.
		
		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access.
	 	@param feature Feature to get array design block from. 
		@return An {@link ArrayDesignBlock} object.
		@throws PermissionDeniedException If the logged in user doesn't 
			have read permission to the item
		@throws BaseException If there is another error
	*/
	public static ArrayDesignBlock getArrayDesignBlock(DbControl dc, FeatureData feature)
		throws PermissionDeniedException, BaseException
	{
		ArrayDesignBlockData adb = (ArrayDesignBlockData)metaData.getPropertyValue(feature, "arrayDesignBlock", EntityMode.POJO);
		adb = HibernateUtil.loadData(dc.getHibernateSession(), ArrayDesignBlockData.class, adb.getId());
		return dc.getItem(ArrayDesignBlock.class, adb);
	}

	/**
		Get the well from a feature.
		
		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access.
	 	@param feature The feature to get well from. 
		@return A {@link Well} object or null if no well is connected
		@throws PermissionDeniedException If the logged in user doesn't 
			have read permission to the item
		@throws BaseException If there is another error
	*/
	public static Well getWell(DbControl dc, FeatureData feature)
		throws PermissionDeniedException, BaseException
	{
		WellData well = (WellData)metaData.getPropertyValue(feature, "well", EntityMode.POJO);
		return well == null ? null : dc.getItem(Well.class, 
			HibernateUtil.loadData(dc.getHibernateSession(), WellData.class, well.getId()));
	}
	
	/**
		Get the coordinate of the feature.
		@param feature
		@return A {@link FeatureCoordinate} object 
	*/
	public static FeatureCoordinate getFeatureCoordinate(FeatureData feature)
	{
		ArrayDesignBlockData adb = (ArrayDesignBlockData)metaData.getPropertyValue(feature, "arrayDesignBlock", EntityMode.POJO);
		return new FeatureCoordinate(adb.getBlockNumber(), adb.getMetaGridX(), adb.getMetaGridY(), 
			feature.getRow(), feature.getColumn());
	}

}
