/**
	$Id: SimpleAbsoluteProgressReporter.java 4517 2008-09-11 20:11:02Z jari $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

/**
	This is a simple implementation of the {@link AbsoluteProgressReporter} interface,
 	which simply remembers that last values. 
 	
	@author nicklas
	@version 2.0
	@base.modified $Date: 2008-09-11 22:11:02 +0200 (Thu, 11 Sep 2008) $
	@see SimpleProgressReporter
*/
public class SimpleAbsoluteProgressReporter
	extends SimpleProgressReporter
	implements AbsoluteProgressReporter
{

	private final long total;
	private long completed;
	
	/**
		Create a new absolute progress reporter.
		@param progress An optional progress which is also notified about
			the progress
		@param total The end point of task
	*/
	public SimpleAbsoluteProgressReporter(ProgressReporter progress, long total)
	{
		super(progress);
		this.total = total;
	}
	
	/*
		From the AbsoluteProgressReporter interface
		-------------------------------------------
	*/	
	public void displayAbsolute(long completed, String message)
	{
		this.completed = completed;
		display((int)(100 * completed / total), message);
	}
	// -------------------------------------------

	/**
		Get the end point of the task.
	*/
	public long getTotal()
	{
		return total;
	}
	/**
		Get completed number of ticks of the task.
	*/
	public long getCompleted()
	{
		return completed;
	}
}
