/*
	$Id: ItemKeyData.java 4517 2008-09-11 20:11:02Z jari $

	Copyright (C) 2005, 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

import java.util.Map;
import java.util.HashMap;

/**
	This class holds information access privileges for users and groups.

	@author Nicklas
	@version 2.0
	@see net.sf.basedb.core.ItemKey
	@see <a href="../../../../../../html/developerdoc/api/data_api.authentication.html">Authentication overview</a>
	@base.modified $Date: 2008-09-11 22:11:02 +0200 (Thu, 11 Sep 2008) $
	@hibernate.subclass discriminator-value="1"
*/
public class ItemKeyData
	extends KeyData
{

	public ItemKeyData()
	{}

	private Map<UserData,Integer> users;
	/**
		Get the map that manages which users that have permissions 
		for this key.

		@hibernate.map table="`UserKeys`" lazy="true"
		@hibernate.collection-key column="`key_id`"
		@hibernate.index-many-to-many column="`user_id`" class="net.sf.basedb.core.data.UserData"
		@hibernate.collection-element column="`permission`" type="int" not-null="true"
	*/
	public Map<UserData,Integer> getUsers()
	{
		if (users == null) users = new HashMap<UserData,Integer>();
		return users;
	}
	void setUsers(Map<UserData,Integer> users)
	{
		this.users = users;
	}

	private Map<GroupData,Integer> groups;
	/**
		Get the map that manages which groups that have permissions 
		for this key.

		@hibernate.map table="`GroupKeys`" lazy="true"
		@hibernate.collection-key column="`key_id`"
		@hibernate.index-many-to-many column="`group_id`" class="net.sf.basedb.core.data.GroupData"
		@hibernate.collection-element column="`permission`" type="int" not-null="true"
	*/
	public Map<GroupData,Integer> getGroups()
	{
		if (groups == null) groups = new HashMap<GroupData,Integer>();
		return groups;
	}
	void setGroups(Map<GroupData,Integer> groups)
	{
		this.groups = groups;
	}

}
