/*
	$Id: PluginConfigurationData.java 4517 2008-09-11 20:11:02Z jari $

	Copyright (C) 2005 Samuel Andersson, Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

import java.util.Map;
import java.util.HashMap;

/**
	This data class is used to configure a plugin. A plugin can have many
	configurations.

	@author Nicklas, Samuel
	@version 2.0
	@see net.sf.basedb.core.PluginConfiguration
	@see <a href="../../../../../../html/developerdoc/api/data_api.plugins.html">Plugins</a>
	@hibernate.class table="`PluginConfigurations`" lazy="false"
*/
public class PluginConfigurationData
	extends AnnotatedData
{
	public PluginConfigurationData()
	{}
	
	private PluginDefinitionData pluginDefinition;
	/**
		Get the plugin definition for this configuration.
		@return A <code>PluginDefinition</code> object.
		@hibernate.many-to-one column="`plugindefinition_id`" not-null="true" update="false"
	*/
	public PluginDefinitionData getPluginDefinition()
	{
		return pluginDefinition;
	}
	public void setPluginDefinition(PluginDefinitionData pd)
	{
		this.pluginDefinition = pd;
	}

	private int parameterVersion;
	/**
		Get the current/latest parameter version.
		@hibernate.property column="`parameter_version`" type="int" not-null="true"
	*/
	public int getParameterVersion()
	{
		return parameterVersion;
	}
	public void setParameterVersion(int parameterVersion)
	{
		this.parameterVersion = parameterVersion;
	}

	private Map<VersionedParameter, ParameterValueData<?>> configurationValues;
	/**
		Used by Hibernate to link with configuration values.
		@hibernate.map table="`VersionedPluginConfigurationValues`" lazy="true" cascade="all"
		@hibernate.collection-key column="`pluginconfiguration_id`"
		@hibernate.collection-composite-index class="net.sf.basedb.core.data.VersionedParameter"
		@hibernate.collection-many-to-many column="`value_id`" class="net.sf.basedb.core.data.ParameterValueData"
	*/
	public Map<VersionedParameter, ParameterValueData<?>> getConfigurationValues()
	{
		if (configurationValues == null) 
		{
			configurationValues = new HashMap<VersionedParameter, ParameterValueData<?>>();
		}
		return configurationValues;
	}
	void setConfigurationValues(Map<VersionedParameter, ParameterValueData<?>> configurationValues)
	{
		this.configurationValues = configurationValues;
	}
}
