/*
	$Id: ProjectData.java 4517 2008-09-11 20:11:02Z jari $

	Copyright (C) 2005 Samuel Andersson, Jari Hakkinen, Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

import java.util.Set;
import java.util.Map;
import java.util.HashMap;

/**
	This class holds information about a project.

	@author Nicklas
	@version 2.0
	@see net.sf.basedb.core.Project
	@see <a href="../../../../../../html/developerdoc/api/data_api.authentication.html">Authentication overview</a>
	@base.modified $Date: 2008-09-11 22:11:02 +0200 (Thu, 11 Sep 2008) $
	@hibernate.class table="`Projects`" lazy="false"
*/
public class ProjectData
	extends OwnedData
	implements NameableData, RemovableData
{

	public ProjectData()
	{}
	
	/*
		From the NameableData interface
		-------------------------------------------
	*/
	private String name;
	public String getName()
	{
		return name;
	}
	public void setName(String name)
	{
		this.name = name;
	}
	private String description;
	public String getDescription()
	{
		return description;
	}
	public void setDescription(String description)
	{
		this.description = description;
	}
	// -------------------------------------------
	/*
		From the RemovableData interface
		-------------------------------------------
	*/
	private boolean removed;
	public boolean isRemoved()
	{
		return removed;
	}
	public void setRemoved(boolean removed)
	{
		this.removed = removed;
	}
	// -------------------------------------------

	private Map<UserData,Integer> users;
	/**
		Get the map that manages which users are members of this project 
		and their permissions.
		@hibernate.map table="`UserProjects`" lazy="true"
		@hibernate.collection-key column="`project_id`"
		@hibernate.index-many-to-many column="`user_id`" class="net.sf.basedb.core.data.UserData"
		@hibernate.collection-element column="`permission`" type="int" not-null="true"
	*/
	public Map<UserData,Integer> getUsers()
	{
		if (users == null) users = new HashMap<UserData,Integer>();
		return users;
	}
	void setUsers(Map<UserData,Integer> users)
	{
		this.users = users;
	}

	private Map<GroupData,Integer> groups;
	/**
		Get the map that manages which users are members of this project 
		and their permissions.
		@hibernate.map table="`GroupProjects`" lazy="true"
		@hibernate.collection-key column="`project_id`"
		@hibernate.index-many-to-many column="`group_id`" class="net.sf.basedb.core.data.GroupData"
		@hibernate.collection-element column="`permission`" type="int" not-null="true"
	*/
	public Map<GroupData,Integer> getGroups()
	{
		if (groups == null) groups = new HashMap<GroupData,Integer>();
		return groups;
	}
	void setGroups(Map<GroupData,Integer> groups)
	{
		this.groups = groups;
	}
	
	private Set<ProjectKeyData> projectKeys;
	/**
		This is the inverse end.
		@see ProjectKeyData#getProjects()
		@hibernate.set table="`ProjectKeys`" lazy="true"
		@hibernate.collection-key column="`project_id`"
		@hibernate.collection-many-to-many column="`key_id`" class="net.sf.basedb.core.data.ProjectKeyData"
	*/
	Set<ProjectKeyData> getProjectKeys()
	{
		return projectKeys;
	}
	void setProjectKeys(Set<ProjectKeyData> projectKeys)
	{
		this.projectKeys = projectKeys;
	}
	
	private Map<String, String> projectDefaults;
	/**
	  	Gets the default values 
	  	@hibernate.map table="`ProjectDefaults`" lazy="true"
	  	@hibernate.collection-key column="`project_id`"
	  	@hibernate.collection-index column="`name`" type="string" length="255" not-null="true"
	  	@hibernate.collection-element column="`value`" type="string" length="255" not-null="true"
	*/
	public Map<String, String> getProjectDefaults()
	{
		if (projectDefaults == null)projectDefaults = new HashMap<String,String>();
		return projectDefaults;
	}
	void setProjectDefaults(Map<String, String> projectDefaults)
	{
		this.projectDefaults = projectDefaults;
	}
}
