/**
	$Id: IntegerFormatter.java 4515 2008-09-11 20:08:14Z jari $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.formatter;


/**
 	Formats a number as an integer. 
 	
	@author nicklas
	@version 2.2
	@base.modified $Date: 2008-09-11 22:08:14 +0200 (Thu, 11 Sep 2008) $
*/
public class IntegerFormatter
	implements Formatter<Number>
{

	private final boolean longValues;
	
	/**
		Create a new integer formatter.
		@param longValues If the {@link #parseString(String)} method should return long
			or int values
	*/
	public IntegerFormatter(boolean longValues)
	{
		this.longValues = longValues;
	}
	
	/*
		From the Formatter interface
		-------------------------------------------
	*/
	public String format(Number value)
	{
		return value == null ? "" : String.valueOf(value.longValue());
	}
	public Number parseString(String value)
	{
		Double d = Double.valueOf(value);
		Number n = null;
		if (longValues)
		{
			n = Long.valueOf(d.longValue());
		}
		else
		{
			n = Integer.valueOf(d.intValue());
		}
		return n;
	}
	// -------------------------------------------

}
