/*
	$Id: TestBioSource.java 4514 2008-09-11 20:07:12Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import net.sf.basedb.core.*;

import java.util.Date;

public class TestBioSource
{

	static boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing biosources");
		write_header();
		// Standard tests: create, load, list
		int id = test_create(true);
		int id2 = test_create(false);
		test_load(id);
		test_list(-1);

		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		// Standard test: Delete
		test_delete(id);
		test_delete(id2);
		write("++Testing biosources "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static int test_create(boolean setAll)
	{
		if (!TestUtil.hasPermission(Permission.CREATE, Item.BIOSOURCE)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			BioSource b = BioSource.getNew(dc);
			if (setAll)
			{
				b.setName("Test biosource");
				b.setDescription("Added at "+new Date());
				b.setExternalId("XCV-455");
			}
			dc.saveItem(b);
			dc.commit();
			id = b.getId();
			write_item(0, b);
			write("--Create biosource OK");
		}
		catch (Throwable ex)
		{
			write("--Create biosource FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			BioSource b = BioSource.getById(dc, id);
			write_item(0, b);
			write("--Load biosource OK");
		}
		catch (Throwable ex)
		{
			write("--Load biosource FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_list(int expectedResults)
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemResultList<BioSource> l = BioSource.getQuery().list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List biosources OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List biosources FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_delete(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			BioSource b = BioSource.getById(dc, id);
			dc.deleteItem(b);
			dc.commit();
			write("--Delete biosource OK");
		}
		catch (Throwable ex)
		{
			write("--Delete biosource FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_delete_all()
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemResultList<BioSource> l = BioSource.getQuery().list(dc);
			Trashcan.delete(dc.getSessionControl(), l, true, null);
			dc.commit();
			write("--Delete all biosources OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--Delete all biosources FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tName      \tDescription\tExternal id");
			write("-- \t-- \t--------- \t-----------\t-----------");
		}
	}
	static void write_item(int i, BioSource b)
		throws BaseException
	{
		if (!TestUtil.getSilent()) System.out.println(i+":\t"+b.getId()+"\t"+b.getName()+"\t"+b.getDescription()+
			"\t"+b.getExternalId());
	}
	static void write(String message)
	{
		System.out.println(message);
	}
}
