/*
	$Id: TestExperiment.java 4544 2008-09-25 07:01:30Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Johan Enell, Jari Hakkinen, Nicklas Nordborg, Gregory Vincic
	Copyright (C) 2007 Johan Enell, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import net.sf.basedb.core.*;
import net.sf.basedb.core.query.Aggregations;
import net.sf.basedb.core.query.Restrictions;
import net.sf.basedb.core.query.Expressions;
import net.sf.basedb.core.query.Expression;
import net.sf.basedb.core.query.Dynamic;
import net.sf.basedb.core.query.SqlResult;
import net.sf.basedb.core.query.JoinType;
import net.sf.basedb.core.query.Selects;

import net.sf.basedb.util.IntensityCalculatorUtil;
import net.sf.basedb.util.IntensityCalculator;
import net.sf.basedb.util.jep.Jep;
import net.sf.basedb.util.jep.RawFunction;
import net.sf.basedb.util.ConsoleProgressReporter;

import java.io.PrintWriter;
import java.util.Collections;
import java.util.Date;
import java.util.Set;
import java.util.List;
import java.util.LinkedList;
import java.util.ArrayList;
import java.util.HashSet;


public class TestExperiment
{

	static boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing experiments");
		write_header();

		// Standard tests: create, load, list
		int id = test_create("genepix", true);
		int id2 = test_create("genepix", false);
		
		test_load(id);
		test_list(-1);
		
		// Experimental factors
		int factorId1 = TestAnnotationType.test_create("Drug resistance", Type.STRING, 0, null, null, Item.RAWBIOASSAY, 1, new String[] { "high", "medium", "low" }, false);
		int factorId2 = TestAnnotationType.test_create("Temperature", Type.FLOAT, 0, null, null, Item.RAWBIOASSAY, 1, null, false);
		test_add_experimental_factors(id, factorId1, factorId2);
		test_list_experimental_factors(id, 2);
		
		// Import reporters
		TestReporter.test_import_from_file("data/test.rawdata.import.txt", 
			"\"Block\"\\t\"Column\"\\t\"Row\"\\t\"Name\"\\t\"ID\".*", "\\t", 4, 3);
		
		int arrayDesignId = TestArrayDesign.test_create(Platform.GENERIC, false);
		TestArrayDesign.test_import_from_file(arrayDesignId, "data/test.rawdata.import.txt", 
			"\"Block\"\\t\"Column\"\\t\"Row\"\\t\"Name\"\\t\"ID\".*", null, 0, 2, 1, 4);
		
		int rbaId1 = TestRawBioAssay.test_create(Platform.GENERIC, "genepix", 0, 0, 0, arrayDesignId, false);
		int rbaId2 = TestRawBioAssay.test_create(Platform.GENERIC, "genepix", 0, 0, 0, arrayDesignId, false);
		TestRawBioAssay.test_import_from_file(rbaId1, "data/test.rawdata.import.txt");
		TestRawBioAssay.test_import_from_file(rbaId2, "data/test.rawdata.import.txt");

		test_add_rawbioassay(id, rbaId1);
		test_add_rawbioassay(id, rbaId2);
		test_list_rawbioassays(id, 2);

		int emptyRootId = test_create_empty_root_bioassayset(id, rbaId1, rbaId2);
		
		int rootBasId = test_create_root_bioassayset_using_calculator(id, "mean", rbaId1, rbaId2);
		int rootBasId2 = test_create_root_bioassayset_using_query(id, "mean", rbaId1, rbaId2);
		int fileBasId = test_create_file_bioassayset(rootBasId, "/spotdata-" + rootBasId + ".txt");
		
		
		int formulaId = TestFormula.test_create(null, Formula.Type.INTENSITY_EXPRESSION,
				"genepix", new String[] { "raw('ch1FgMean')", "raw('ch2FgMean')" }, false);
		int jobId = test_create_root_bioassayset_using_plugin(id, formulaId, rbaId1, rbaId2);
		TestJob.test_execute(jobId);
		
		
		int filterBasId = test_filter_bioassayset(rootBasId, 2.0f);
		int basId = test_create_bioassayset(filterBasId);
		
		int intExtraId = TestExtraValueType.test_create("test.int", Type.INT, false);
		int floatExtraId = TestExtraValueType.test_create("test.float", Type.FLOAT, false);
		int stringExtraId = TestExtraValueType.test_create("test.string", Type.STRING, false);
		
		int extraSpotId = test_calculate_spotextravalues(filterBasId, true,
				intExtraId, floatExtraId, stringExtraId);
		int extraPosId = test_calculate_positionextravalues(filterBasId, intExtraId, floatExtraId, stringExtraId);
		// TODO 
		//int mergedBasId = test_merge_bioassays(filterBasId);
		
		test_list_bioassaysets(id, 9);
		
		test_list_bioassays(basId, 2);
		
		write_data_header();
		test_list_bioassayset_data(extraSpotId, 10, 2, intExtraId, floatExtraId, stringExtraId);
		
		write_extra_values_header();
		test_list_extra_values(extraSpotId, intExtraId, 10, 2);
		
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		
		test_delete_bioassayset(filterBasId);
		test_delete_data_files(fileBasId);
		
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		
		test_remove_rawbioassay(id, rbaId1);
		test_remove_rawbioassay(id, rbaId2);

		// Standard test: Delete
		test_delete(id);
		test_delete(id2);

		TestAnnotationType.test_delete(factorId1);
		TestAnnotationType.test_delete(factorId2);
		TestExtraValueType.test_delete(intExtraId);
		TestExtraValueType.test_delete(floatExtraId);
		TestExtraValueType.test_delete(stringExtraId);
		TestRawBioAssay.test_delete(rbaId1);
		TestRawBioAssay.test_delete(rbaId2);
		TestArrayDesign.test_delete(arrayDesignId);
		TestReporter.test_delete();
		TestFormula.test_delete(formulaId);
		write("++Testing experiments "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static int test_create(String rawDataType, boolean setAll)
	{
		if (!TestUtil.hasPermission(Permission.CREATE, Item.EXPERIMENT)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Experiment e = Experiment.getNew(dc, RawDataTypes.getSafeRawDataType(rawDataType));
			if (setAll)
			{
				e.setName("Test experiment");
				e.setDescription("Added at "+new Date());
				e.setAbstract("An abstract");
				e.setAffiliations("The affiliations");
				e.setAuthors("The authors");
				e.setExperimentDesign("A cool design");
				e.setExperimentType("The type");
				e.setPublication("Here is the publication");
				e.setPublicationDate(new Date());
				e.setPubMedId("PUBMED-123ABC");
			}
			dc.saveItem(e);
			dc.commit();
			id = e.getId();
			dc = TestUtil.getDbControl();
			dc.reattachItem(e);
			write_item(0, e);
			write("--Create experiment OK");
		}
		catch (Throwable ex)
		{
			write("--Create experiment FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Experiment e = Experiment.getById(dc, id);
			write_item(0, e);
			write("--Load experiment OK");
		}
		catch (Throwable ex)
		{
			write("--Load experiment FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_list(int expectedResults)
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemResultList<Experiment> l = Experiment.getQuery().list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List experiments OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List experiments FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_delete(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Experiment e = Experiment.getById(dc, id);
			dc.deleteItem(e);
			dc.commit();
			write("--Delete experiment OK");
		}
		catch (Throwable ex)
		{
			write("--Delete experiment FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tName      \tRaw data type\tDescription\tAbstract\tAffiliations\tAuthors\tDesign\tType\tPublication\tDate\tPubMed ID");
			write("-- \t-- \t--------- \t-------------\t-----------\t--------\t------------\t-------\t------\t----\t-----------\t----\t---------");
		}
	}
	static void write_item(int i, Experiment e)
		throws BaseException
	{
		if (!TestUtil.getSilent()) 
		{
			System.out.println(i+":\t"+e.getId()+"\t"+e.getName()+"\t"+e.getRawDataType().getName()+"\t"+e.getDescription()+
				"\t"+e.getAbstract()+"\t"+e.getAffiliations()+"\t"+e.getAuthors()+
				"\t"+e.getExperimentDesign()+"\t"+e.getExperimentType()+"\t"+e.getPublication()+
				"\t"+e.getPublicationDate()+"\t"+e.getPubMedId());
		}
	}
	static void write_item(int i, RawBioAssay rba)
		throws BaseException
	{
		if (!TestUtil.getSilent()) 
		{
			System.out.println(i+":\t"+rba.getId()+"\t"+rba.getName()+"\t"+rba.getDescription()+
				"\t"+rba.getRawDataType().getName()+"\t"+rba.getNumDbSpots()+"\t"+rba.getBytes()+"\t"+rba.getArrayDesign()+
				"\t"+rba.getSoftware()+"\t"+rba.getProtocol());
		}
	}
	static void write_item(int i, BioAssay ba)
		throws BaseException
	{
		if (!TestUtil.getSilent()) 
		{
			System.out.println(i+":\t"+ba.getId()+"\t"+ba.getName()+"\t"+ba.getDescription()+
				"\t"+ba.getRawDataType().getName()+"\t"+ba.getNumSpots());
		}
	}
	static void write_item(int i, BioAssaySet bas)
		throws BaseException
	{
		if (!TestUtil.getSilent()) 
		{
			System.out.println(i+":\t"+bas.getId()+"\t"+bas.getName()+"\t"+bas.getDescription()+
				"\t"+bas.getTransformation()+"\t"+bas.getNumReporters()+"\t"+bas.getNumSpots() +
				"\t" + bas.getNumFileReporters() + "\t" + bas.getNumFileSpots());
		}
	}
	static void write_item(int i, ExtraValue ev)
		throws BaseException
	{
		if (!TestUtil.getSilent()) 
		{
			System.out.println("\tExtra value: "+i+":\t"+ev.getId()+"\t"+ev.getValueType()+"\t"+
				ev.getCoordinateType()+"\t"+ev.getExtraValueType().getExternalId()+"\t"+ev.getNumValues());
		}
	}
	static void write_data_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tColumn\tPosition\tChannel 1\tChannel 2\tReporter\tX  \tY  \tExtra");
			write("-- \t------\t--------\t---------\t---------\t--------\t---\t---\t-----");
		}
	}
	static void write_extra_values_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tColumn\tPosition\tExtra");
			write("-- \t------\t--------\t-----");
		}
	}
	static void write_data(int i, short column, int position, float ch1, float ch2, 
		String reporter, float x, float y, List<Object> extra)
	{
		if (!TestUtil.getSilent())
		{
			System.out.println(i+":\t"+column+"\t"+position+"\t"+ch1+"\t"+ch2+"\t"+reporter+
				"\t"+x+"\t"+y+"\t"+extra);
		}
	}
	static void write_extra_value(int i, short column, int position, Object extra)
	{
		if (!TestUtil.getSilent())
		{
			System.out.println(i+":\t"+column+"\t"+position+"\t"+extra);
		}
	}
	
	static void write(String message)
	{
		System.out.println(message);
	}
	
	static void test_add_rawbioassay(int id, int rawBioAssayId)
	{
		if (id == 0 || rawBioAssayId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Experiment e = Experiment.getById(dc, id);
			RawBioAssay rba = RawBioAssay.getById(dc, rawBioAssayId);
			e.addRawBioAssay(rba);
			dc.commit();
			write("--Add raw bioassay OK");
		}
		catch (Throwable ex)
		{
			write("--Add raw bioassay FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_remove_rawbioassay(int id, int rawBioAssayId)
	{
		if (id == 0 || rawBioAssayId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Experiment e = Experiment.getById(dc, id);
			RawBioAssay rba = RawBioAssay.getById(dc, rawBioAssayId);
			e.removeRawBioAssay(rba);
			dc.commit();
			write("--Remove raw bioassay OK");
		}
		catch (Throwable ex)
		{
			write("--Remove raw bioassay FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static int[] test_list_rawbioassays(int id, int expectedResults)
	{
		if (id == 0) return null;
		DbControl dc = null;
		int[] rawBioAssays = null;
		try
		{
			dc = TestUtil.getDbControl();
			Experiment e = Experiment.getById(dc, id);

			ItemResultList<RawBioAssay> l = e.getRawBioAssays().list(dc);
			rawBioAssays = new int[l.size()];
			for (int i = 0; i<l.size(); i++)
			{
				rawBioAssays[i] = l.get(i).getId();
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List raw bioassays OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List raw bioassays FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return rawBioAssays;
	}
	
	static void test_add_experimental_factors(int id, int... annotationTypes)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Experiment e = Experiment.getById(dc, id);
			for (int atId : annotationTypes)
			{
				AnnotationType at = AnnotationType.getById(dc, atId);
				e.addExperimentalFactor(at);
			}
			dc.commit();
			write("--Add experimental factors OK");
		}
		catch (Throwable ex)
		{
			write("--Add experimental factors FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void test_remove_experimental_factors(int id, int... annotationTypes)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Experiment e = Experiment.getById(dc, id);
			for (int atId : annotationTypes)
			{
				AnnotationType at = AnnotationType.getById(dc, atId);
				e.removeExperimentalFactor(at);
			}
			dc.commit();
			write("--Remove experimental factors OK");
		}
		catch (Throwable ex)
		{
			write("--Remove experimental factors FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void test_list_experimental_factors(int id, int expectedResults)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Experiment e = Experiment.getById(dc, id);

			ItemResultList<AnnotationType> l = e.getExperimentalFactors().list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				TestAnnotationType.write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List experimental factors OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List experimental factors FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}	
	
	static int test_create_root_bioassayset_using_plugin(int experimentId, int formulaId, int... rawBioAssays)
	{
		if (experimentId == 0) return 0;
		DbControl dc = null;
		int id = 0;
		try
		{
			dc = TestUtil.getDbControl();
			int pluginDefId = TestPluginDefinition.test_get("net.sf.basedb.plugins.IntensityCalculatorPlugin");
			PluginDefinition pluginDef = PluginDefinition.getById(dc,pluginDefId);
			Experiment experiment = Experiment.getById(dc, experimentId);
			if( pluginDef != null )
			{
				Job job = Job.getNew(dc, pluginDef, null, experiment);

				/* name parameter */
				StringParameterType stringParameterType = new StringParameterType(255, null, true);
				job.setParameterValue("name", stringParameterType, "Plugin test");

				/* experiment parameter */
				ItemParameterType<Experiment> experimentParameterType = 
					new ItemParameterType<Experiment>(Experiment.class, null, true, 0, null);
				job.setParameterValue("experiment", experimentParameterType, experiment);

				/* rawBioAssays parameter */
				List<RawBioAssay> sources = new LinkedList<RawBioAssay>();
				for (int rbaId : rawBioAssays)
				{
					sources.add(RawBioAssay.getById(dc, rbaId));	
				}
				ItemParameterType<RawBioAssay> rawBioAssayParameterType = 
					new ItemParameterType<RawBioAssay>(RawBioAssay.class, null, true, 0, null);
				job.setParameterValues("rawBioAssays", rawBioAssayParameterType, sources);


				/* formula parameter */
				ItemParameterType<Formula> formulaParameterType = 
					new ItemParameterType<Formula>(Formula.class, null, true, 0, null);
				job.setParameterValue("formula", formulaParameterType, Formula.getById(dc, formulaId));

				job.setPriority(1);

				dc.saveItem(job);
				dc.commit();
				id = job.getId();

				write("--Create root bioassayset using plugin OK");

			}
			

		}
		catch (Throwable ex)
		{
			write("--Create root bioassayset using plugin FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}


	static int test_create_root_bioassayset_using_calculator(int experimentId, String intensityFormulaName, int... rawBioAssays)
	{
		if (experimentId == 0) return 0;
		int id = 0;
		DbControl dc = null;
		long time;
		try
		{
			dc = TestUtil.getDbControl();
			Experiment e = Experiment.getById(dc, experimentId);
			
			Set<RawBioAssay> rbaCollection = new HashSet<RawBioAssay>();
			for (int rbaId : rawBioAssays)
			{
				rbaCollection.add(RawBioAssay.getById(dc, rbaId));	
			}
			
			RawDataType rdt = e.getRawDataType();
			IntensityCalculator iCalc = IntensityCalculatorUtil.createJepIntensityCalculator(
				dc, rdt, rdt.getIntensityFormula(intensityFormulaName).getExpressions());
			
			time = System.currentTimeMillis();
			BioAssaySet root = IntensityCalculatorUtil.createRootBioAssaySet(
				dc, e, rbaCollection, null, iCalc, TestUtil.getSilent() ? null : new ConsoleProgressReporter());
			dc.commit();

			time = System.currentTimeMillis()-time;
			int numSpots = root.getNumSpots();
			id = root.getId();
			dc = TestUtil.getDbControl();
			dc.reattachItem(root);
			write_item(0, root);
			write("--Create root bioassayset using calculator OK ("+numSpots+" spots inserted; "+time+" ms)");
		}
		catch (Throwable ex)
		{
			write("--Create root bioassayset using calculator FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}
	
	static int test_create_root_bioassayset_using_query(int experimentId, String intensityFormulaName, int... rawBioAssays)
	{
		if (experimentId == 0) return 0;
		int id = 0;
		DbControl dc = null;
		long time;
		try
		{
			dc = TestUtil.getDbControl();
			Experiment e = Experiment.getById(dc, experimentId);
			
			Set<RawBioAssay> rbaCollection = new HashSet<RawBioAssay>();
			for (int rbaId : rawBioAssays)
			{
				rbaCollection.add(RawBioAssay.getById(dc, rbaId));	
			}
			
			RawDataType rdt = e.getRawDataType();
			String[] formulas = rdt.getIntensityFormula(intensityFormulaName).getExpressions();
			RawFunction raw = new RawFunction(dc, e.getRawDataType(), true);
			Expression ch1 = Jep.formulaToExpression(formulas[0], raw);
			Expression ch2 = Jep.formulaToExpression(formulas[1], raw);

			time = System.currentTimeMillis();
			BioAssaySet root = IntensityCalculatorUtil.createRootBioAssaySet(
				dc, e, rbaCollection, null, new Expression[] { ch1, ch2 }, 
				TestUtil.getSilent() ? null : new ConsoleProgressReporter() );
			dc.commit();

			time = System.currentTimeMillis()-time;
			int numSpots = root.getNumSpots();
			id = root.getId();
			dc = TestUtil.getDbControl();
			dc.reattachItem(root);
			write_item(0, root);
			write("--Create root bioassayset using query OK ("+numSpots+" spots inserted; "+time+" ms)");
		}
		catch (Throwable ex)
		{
			write("--Create root bioassayset using query FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}
	
	static int test_create_empty_root_bioassayset(int experimentId, int... rawBioAssays)
	{
		if (experimentId == 0) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Experiment e = Experiment.getById(dc, experimentId);
			
			Set<RawBioAssay> rbaCollection = new HashSet<RawBioAssay>();
			for (int rbaId : rawBioAssays)
			{
				rbaCollection.add(RawBioAssay.getById(dc, rbaId));	
			}
			
			Transformation t = e.newTransformation(null, rbaCollection);
			BioAssaySet root = t.newProduct("new", "new", false);
			root.setName("No spots");
			dc.saveItem(t);
			dc.saveItem(root);
			
			for (RawBioAssay rba : rbaCollection)
			{
				// Create a new BioAssay for each raw bioassay
				BioAssay ba = root.newRootBioAssay(Collections.singleton(rba));
				ba.setName(rba.getName());
				dc.saveItem(ba);
			}
			dc.commit();

			int numSpots = root.getNumSpots();
			id = root.getId();
			dc = TestUtil.getDbControl();
			dc.reattachItem(root);
			write_item(0, root);
			write("--Create empty root bioassayset OK ("+numSpots+" spots inserted)");
		}
		catch (Throwable ex)
		{
			write("--Create empty root bioassayset FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}
	
	
	static int test_create_file_bioassayset(int bioAssaySetId, String path)
	{
		if (bioAssaySetId == 0) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			
			BioAssaySet source = BioAssaySet.getById(dc, bioAssaySetId);
			Transformation t = source.newTransformation(null);
			dc.saveItem(t);

			BioAssaySet bas = t.newProduct(null, "new", false);
			bas.setName("File bioassayset");
			dc.saveItem(bas);

			for (BioAssay parentBa : source.getBioAssays().list(dc))
			{
				BioAssay childBa = bas.newBioAssay(parentBa);
				childBa.setNumFileSpots(parentBa.getNumSpots());
				dc.saveItem(childBa);
			}
			
			File f = File.getByPath(dc, new Path(path, Path.Type.FILE), true);
			f.setMimeType("text/plain");
			f.setFileType(FileType.getById(dc, SystemItems.getId(FileType.SPOT_DATA)));
			if (!f.isInDatabase()) dc.saveItem(f);
			FileStoreUtil.setDataFile(dc, bas, DataFileType.GENERIC_SPOT_DATA, f);
			bas.setNumFileSpots(source.getNumSpots());
			bas.setNumFileReporters(source.getNumReporters());
			
			PrintWriter out = new PrintWriter(f.getUploadStream(false));
			int numChannels = bas.getRawDataType().getChannels();
			out.print("REPORTER\tASSAY\tPOSITION");
			for (int ch = 1; ch <= numChannels; ++ch)
			{
				out.print("\tCH");
				out.print(ch);
			}
			out.print('\n');
			
			DynamicSpotQuery query = source.getSpotData();
			query.select(Dynamic.selectReporter("externalId"));
			query.select(Dynamic.select(VirtualColumn.COLUMN));
			query.select(Dynamic.select(VirtualColumn.POSITION));
			for (int ch = 1; ch <= numChannels; ++ch)
			{
				query.select(Dynamic.select(VirtualColumn.channel(ch)));
			}
			DynamicResultIterator it = query.iterate(dc);

			while (it.hasNext())
			{
				SqlResult result = it.next();
				out.print(result.getString(1)); // Reporter
				out.print('\t');
				out.print(result.getInt(2)); // Assay
				out.print('\t');
				out.print(result.getInt(3)); // Position
				
				for (int ch = 1; ch <= numChannels; ++ch)
				{
					out.print('\t');
					out.print(result.getFloat(3+ch));
				}
				out.print('\n');
			}
			out.flush();
			out.close();

			dc.commit();
			id = bas.getId();
			dc = TestUtil.getDbControl();
			dc.reattachItem(bas);
			write_item(0, bas);
			write("--Create file bioassayset OK ("+bas.getNumFileSpots()+" spots inserted)");
		}
		catch (Throwable ex)
		{
			write("--Create file bioassayset FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}
	
	
	static int test_create_bioassayset(int bioAssaySetId)
	{
		if (bioAssaySetId == 0) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			BioAssaySet source = BioAssaySet.getById(dc, bioAssaySetId);
			
			Transformation t = source.newTransformation(null);
			dc.saveItem(t);

			BioAssaySet bas = t.newProduct(null, "new", true);
			bas.setName("Second bioassayset");
			dc.saveItem(bas);

			int numSpots = 0;
			SpotBatcher spotBatcher = bas.getSpotBatcher();
			
			DynamicSpotQuery query = source.getSpotData();
			query.select(Dynamic.select(VirtualColumn.COLUMN));
			query.select(Dynamic.select(VirtualColumn.POSITION));
			query.select(
				Selects.expression(
					Expressions.multiply(
						Dynamic.column(VirtualColumn.channel(1)),
						Expressions.integer(2)
					),
					"ch1"
				)
			);
			query.select(
				Selects.expression(
					Expressions.multiply(
						Dynamic.column(VirtualColumn.channel(2)),
						Expressions.integer(4)
					),
					"ch2"
				)
			);
			spotBatcher.insert(query);
			spotBatcher.close();
			/*
			DynamicResultIterator spots = source.getSpotData().iterate(dc);
			int column = spots.getIndex(VirtualColumn.COLUMN.getName());
			int position = spots.getIndex(VirtualColumn.POSITION.getName());
			int ch1 = spots.getIndex(VirtualColumn.channel(1).getName());
			int ch2 = spots.getIndex(VirtualColumn.channel(2).getName());
			
			while (spots.hasNext())
			{
				SqlResult r = spots.next();
				spotBatcher.insert(r.getShort(column), r.getInt(position), 
					r.getFloat(ch1) * 2, r.getFloat(ch2) * 4);
				numSpots++;
			}
			*/

			dc.commit();
			id = bas.getId();
			dc = TestUtil.getDbControl();
			dc.reattachItem(bas);
			write_item(0, bas);
			write("--Create bioassayset OK ("+numSpots+" spots inserted)");
		}
		catch (Throwable ex)
		{
			write("--Create bioassayset FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static int test_filter_bioassayset(int bioAssaySetId, float factor)
	{
		if (bioAssaySetId == 0) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			BioAssaySet source = BioAssaySet.getById(dc, bioAssaySetId);
			
			Transformation t = source.newTransformation(null);
			dc.saveItem(t);

			BioAssaySet bas = t.newProduct(null, null, true);
			bas.setName("Filtered bioassayset");
			dc.saveItem(bas);

			DynamicSpotQuery query = source.getSpotData();
			query.select(Dynamic.select(VirtualColumn.COLUMN));
			query.select(Dynamic.select(VirtualColumn.POSITION));
			query.restrict(
				Restrictions.gt(
					Dynamic.column(VirtualColumn.channel(1)),
					Expressions.multiply(
						Expressions.parameter("factor"),
						Dynamic.column(VirtualColumn.channel(2))
					)
				)
			);
			query.setParameter("factor", factor, Type.FLOAT);
			
			FilterBatcher filterBatcher = bas.getFilterBatcher();
			DynamicResultIterator spots = query.iterate(dc);
			int column = spots.getIndex(VirtualColumn.COLUMN.getName());
			int position = spots.getIndex(VirtualColumn.POSITION.getName());
			int numSpots = 0;
			while (spots.hasNext())
			{
				SqlResult r = spots.next();
				filterBatcher.insert(r.getShort(column), r.getInt(position));
				numSpots++;
			}
			filterBatcher.close();

			dc.commit();
			id = bas.getId();
			dc = TestUtil.getDbControl();
			dc.reattachItem(bas);
			write_item(0, bas);
			write("--Create filtered bioassayset OK ("+numSpots+" spots inserted)");
		}
		catch (Throwable ex)
		{
			write("--Create filtered bioassayset FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static int test_calculate_spotextravalues(int bioAssaySetId, boolean createChild, 
			int intExtraId, int floatExtraId, int stringExtraId)
	{
		if (bioAssaySetId == 0) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			BioAssaySet source = BioAssaySet.getById(dc, bioAssaySetId);
			
			BioAssaySet insertTo = source;
			if (createChild)
			{
				Transformation t = source.newTransformation(null);
				dc.saveItem(t);
	
				insertTo = t.newProduct(null, null, true);
				insertTo.setName("Extra spot values bioassayset");
				dc.saveItem(insertTo);
			}

			SpotExtraValueBatcher<Integer> intBatcher = null;
			if (intExtraId != 0)
			{
				ExtraValueType intExtraValue = ExtraValueType.getById(dc, intExtraId);
				intBatcher = insertTo.getSpotExtraValueBatcher(Integer.class, intExtraValue, null);
			}
			SpotExtraValueBatcher<Float> floatBatcher = null;
			if (floatExtraId != 0)
			{
				ExtraValueType floatExtraValue = ExtraValueType.getById(dc, floatExtraId);
				floatBatcher = insertTo.getSpotExtraValueBatcher(Float.class, floatExtraValue, null);
			}
			SpotExtraValueBatcher<String> stringBatcher = null;
			if (stringExtraId != 0)
			{
				ExtraValueType stringExtraValue = ExtraValueType.getById(dc, stringExtraId);
				stringBatcher = insertTo.getSpotExtraValueBatcher(String.class, stringExtraValue, null);
			}
			
			DynamicSpotQuery query = source.getSpotData();
			DynamicResultIterator spots = query.iterate(dc);
			int column = spots.getIndex(VirtualColumn.COLUMN.getName());
			int position = spots.getIndex(VirtualColumn.POSITION.getName());
			int ch1 = spots.getIndex(VirtualColumn.channel(1).getName());
			int ch2 = spots.getIndex(VirtualColumn.channel(2).getName());
			int numSpots = 0;
			while (spots.hasNext())
			{
				numSpots++;
				SqlResult r = spots.next();
				
				if (numSpots == 1)
				{
					// Test NULL and Float.NaN
					if (intBatcher != null) intBatcher.insert(r.getShort(column), r.getInt(position), null);
					if (floatBatcher != null) floatBatcher.insert(r.getShort(column), r.getInt(position), Float.NaN);
					if (stringBatcher != null) stringBatcher.insert(r.getShort(column), r.getInt(position), null);
				}
				else
				{
					if (intBatcher != null) intBatcher.insert(r.getShort(column), r.getInt(position), numSpots);
					if (floatBatcher != null) floatBatcher.insert(r.getShort(column), r.getInt(position), r.getFloat(ch1) * r.getFloat(ch2));
					if (stringBatcher != null) stringBatcher.insert(r.getShort(column), r.getInt(position), "Spot number " + numSpots);
				}
			}

			if (intBatcher != null) intBatcher.close();
			if (floatBatcher != null) floatBatcher.close();
			if (stringBatcher != null) stringBatcher.close();

			dc.commit();
			id = insertTo.getId();
			dc = TestUtil.getDbControl();
			dc.reattachItem(insertTo);
			write_item(0, insertTo);
			write("--Calculate spot extra values OK ("+numSpots+" values inserted)");

		}
		catch (Throwable ex)
		{
			write("--Calculate spot extra values FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static int test_calculate_positionextravalues(int bioAssaySetId, int intExtraId, int floatExtraId, int stringExtraId)
	{
		if (bioAssaySetId == 0) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			BioAssaySet source = BioAssaySet.getById(dc, bioAssaySetId);
			
			Transformation t = source.newTransformation(null);
			dc.saveItem(t);

			BioAssaySet bas = t.newProduct(null, null, true);
			bas.setName("Extra position values bioassayset");
			dc.saveItem(bas);
			
			PositionExtraValueBatcher<Integer> intBatcher = null;
			if (intExtraId != 0)
			{
				ExtraValueType intExtraValue = ExtraValueType.getById(dc, intExtraId);
				intBatcher = bas.getPositionExtraValueBatcher(Integer.class, intExtraValue, null);
			}
			PositionExtraValueBatcher<Float> floatBatcher = null;
			if (floatExtraId != 0)
			{
				ExtraValueType floatExtraValue = ExtraValueType.getById(dc, floatExtraId);
				floatBatcher = bas.getPositionExtraValueBatcher(Float.class, floatExtraValue, null);
			}
			PositionExtraValueBatcher<String> stringBatcher = null;
			if (stringExtraId != 0)
			{
				ExtraValueType stringExtraValue = ExtraValueType.getById(dc, stringExtraId);
				stringBatcher = bas.getPositionExtraValueBatcher(String.class, stringExtraValue, null);
			}

			Expression avgCh1 = Aggregations.mean(Dynamic.column(VirtualColumn.channel(1)));
			Expression avgCh2 = Aggregations.mean(Dynamic.column(VirtualColumn.channel(2)));
			Expression product = Expressions.multiply(avgCh1, avgCh2);
			DynamicSpotQuery query = source.getSpotData();
			query.select(Dynamic.select(VirtualColumn.POSITION));
			query.select(Selects.expression(product, "product"));
			query.select(Selects.expression(Expressions.string("static"), "static"));
			query.group(Dynamic.column(VirtualColumn.POSITION));

			DynamicResultIterator spots = query.iterate(dc);
			int position = spots.getIndex(VirtualColumn.POSITION.getName());
			int prod = spots.getIndex("product");
			int stat = spots.getIndex("static");
			int numSpots = 0;
			while (spots.hasNext())
			{
				numSpots++;
				SqlResult r = spots.next();
				if (numSpots == 1)
				{
					if (intBatcher != null) intBatcher.insert(r.getInt(position), null);
					if (floatBatcher != null) floatBatcher.insert(r.getInt(position), Float.NaN);
					if (stringBatcher != null) stringBatcher.insert(r.getInt(position), null);
				}
				else
				{
					if (intBatcher != null) intBatcher.insert(r.getInt(position), numSpots);
					if (floatBatcher != null) floatBatcher.insert(r.getInt(position), r.getFloat(prod));
					if (stringBatcher != null) stringBatcher.insert(r.getInt(position), r.getString(stat));
				}
			}

			if (intBatcher != null) intBatcher.close();
			if (floatBatcher != null) floatBatcher.close();
			if (stringBatcher != null) stringBatcher.close();

			dc.commit();
			id = bas.getId();
			dc = TestUtil.getDbControl();
			dc.reattachItem(bas);
			write_item(0, bas);
			write("--Calculate position extra values OK ("+numSpots+" values inserted)");

		}
		catch (Throwable ex)
		{
			write("--Calculate position extra values FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	
	static void test_delete_bioassayset(int bioAssaySetId)
	{
		if (bioAssaySetId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			BioAssaySet bas = BioAssaySet.getById(dc, bioAssaySetId);
			dc.deleteItem(bas);
			dc.commit();
			write("--Delete bioassayset OK");
		}
		catch (Throwable ex)
		{
			write("--Delete bioassayset FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void test_list_bioassays(int bioassaysetId, int expectedResults)
	{
		if (bioassaysetId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			BioAssaySet bas = BioAssaySet.getById(dc, bioassaysetId);
			
			ItemResultList<BioAssay> l = bas.getBioAssays().list(dc);
			for (int i = 0; i < l.size(); i++)
			{
				BioAssay ba = l.get(i);
				write_item(i, ba);
				for (RawBioAssay rba : ba.getRawBioAssays().list(dc))
				{
					write_item(i, rba);
				}
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List bioassays OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List bioassays FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void test_list_bioassaysets(int experimentId, int expectedResults)
	{
		if (experimentId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Experiment e = Experiment.getById(dc, experimentId);

			ItemResultList<BioAssaySet> l = e.getBioAssaySets().list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				BioAssaySet bas = l.get(i);
				write_item(i, bas);
				
				ItemResultList<ExtraValue> ev = bas.getExtraValues().list(dc);
				
				for (int j = 0; j < ev.size(); j++)
				{
					write_item(j, ev.get(j));	
				}
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List bioassaysets OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List bioassaysets FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void test_list_bioassayset_data(int bioAssaySetId, int limit, int offset, int... extraValueTypes)
	{
		if (bioAssaySetId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			BioAssaySet bas = BioAssaySet.getById(dc, bioAssaySetId);
			
			DynamicSpotQuery query = bas.getSpotData();
			query.select(Dynamic.select(VirtualColumn.COLUMN));
			query.select(Dynamic.select(VirtualColumn.POSITION));
			query.select(Dynamic.select(VirtualColumn.channel(1)));
			query.select(Dynamic.select(VirtualColumn.channel(2)));
			query.select(Dynamic.selectReporter("externalId"));
			query.select(Dynamic.selectRawData("x"));
			query.select(Dynamic.selectRawData("y"));
			query.joinReporters(JoinType.LEFT);
			query.joinRawData(JoinType.LEFT);
			
			for (int extraValueTypeId : extraValueTypes)
			{
				ExtraValueType evt = ExtraValueType.getById(dc, extraValueTypeId);
				ExtraValue ev = bas.getExtraValue(evt);
				query.joinExtraValue(ev, JoinType.INNER);
				query.select(Dynamic.select(ev, "ev"+extraValueTypeId));
			}
			
			query.setMaxResults(limit);
			query.setFirstResult(offset);
			query.setReturnTotalCount(true);
			
			DynamicResultIterator spots = query.iterate(dc);
			int column = spots.getIndex(VirtualColumn.COLUMN.getName());
			int position = spots.getIndex(VirtualColumn.POSITION.getName());
			int ch1 = spots.getIndex(VirtualColumn.channel(1).getName());
			int ch2 = spots.getIndex(VirtualColumn.channel(2).getName());
			int reporter = spots.getIndex("externalId");
			int x = spots.getIndex("x");
			int y = spots.getIndex("y");
			int[] extraIndex = new int[extraValueTypes.length];
			for (int i = 0; i < extraIndex.length; ++i)
			{
				extraIndex[i] = spots.getIndex("ev"+extraValueTypes[i]);	
			}
			List<Object> extraValues = new ArrayList<Object>(extraIndex.length);
			int listed = 0;
			while (spots.hasNext())
			{
				SqlResult r = spots.next();
				for (int i = 0; i < extraIndex.length; ++i)
				{
					extraValues.add(r.getObject(extraIndex[i]));	
				}
				write_data(listed, r.getShort(column), r.getInt(position), 
					r.getFloat(ch1), r.getFloat(ch2), r.getString(reporter), 
					r.getFloat(x), r.getFloat(y), extraValues);
				extraValues.clear();
				listed++;
			}
			spots.close();
			
			dc.close();
			write("--List bioassayset data (listed = " + listed + "; total = " + spots.getTotalCount() + ") OK");
		}
		catch (Throwable ex)
		{
			write("--List bioassayset data FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_list_extra_values(int bioAssaySetId, int extraValueTypeId, int limit, int offset)
	{
		if (bioAssaySetId == 0 || extraValueTypeId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			BioAssaySet bas = BioAssaySet.getById(dc, bioAssaySetId);
			ExtraValueType evt = ExtraValueType.getById(dc, extraValueTypeId);
			ExtraValue ev = bas.getExtraValue(evt);
	
			DynamicExtraValueQuery query = ev.getValues();
			query.setMaxResults(limit);
			query.setFirstResult(offset);
			query.setReturnTotalCount(true);
			
			DynamicResultIterator values = query.iterate(dc);
			int column = values.getIndex(VirtualColumn.COLUMN.getName());
			int position = values.getIndex(VirtualColumn.POSITION.getName());
			int extra = values.getIndex(ev.getVirtualColumn().getName());
			int listed = 0;
			while (values.hasNext())
			{
				SqlResult r = values.next();
				write_extra_value(listed, r.getShort(column), r.getInt(position), 
					r.getObject(extra));
				listed++;
			}
			values.close();
			
			dc.close();
			write("--List extra values (listed = " + listed + "; total = " + values.getTotalCount() + ") OK");
		}
		catch (Throwable ex)
		{
			write("--List extra values FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}

	}

	static void test_delete_data_files(int bioAssaySetId)
	{
		if (bioAssaySetId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			BioAssaySet bas = BioAssaySet.getById(dc, bioAssaySetId);
			List<File> files = new ArrayList<File>();
			if (bas.hasFileSet())
			{
				FileSet fs = bas.getFileSet();
				List<FileSetMember> members = fs.getMembers().list(dc);
				for (FileSetMember fm : members)
				{
					files.add(fm.getFile());
					fs.removeMember(fm.getDataFileType());
				}
			}
			bas.setNumFileReporters(0);
			bas.setNumFileSpots(0);
			dc.commit();
			
			dc = TestUtil.getDbControl();
			for (File f : files)
			{
				dc.deleteItem(f);
			}
			dc.commit();
			write("--Delete data files OK (" + files.size() + ")");
		}
		catch (Throwable ex)
		{
			write("--Delete data files FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
}
