/**
 $Id: TestJobAgent.java 4514 2008-09-11 20:07:12Z jari $

 Copyright (C) 2006 Nicklas Nordborg

 This file is part of BASE - BioArray Software Environment.
 Available at http://base.thep.lu.se/

 BASE is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 3
 of the License, or (at your option) any later version.

 BASE is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

import java.net.InetAddress;
import java.net.Socket;
import java.util.Date;
import java.util.Properties;

import net.sf.basedb.clients.jobagent.Agent;
import net.sf.basedb.clients.jobagent.handlers.DefaultRequestHandler;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemResultList;
import net.sf.basedb.core.Job;
import net.sf.basedb.core.JobAgent;
import net.sf.basedb.core.JobAgentSettings;
import net.sf.basedb.core.Permission;
import net.sf.basedb.core.PluginDefinition;
import net.sf.basedb.core.StringParameterType;
import net.sf.basedb.util.jobagent.JobAgentConnection;
import net.sf.basedb.util.jobagent.RequestHandler;

/**

	@author nicklas
	@version 2.0
	@base.modified $Date: 2008-09-11 22:07:12 +0200 (Thu, 11 Sep 2008) $
*/
public class TestJobAgent
{
	static boolean ok = true;

	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing job agents");
		write_header();
		// Standard tests: create, load, list
		create_fake_jobagent(8888, "net.sf.baseb.clients.agent.test.1");
		create_fake_jobagent(8889, "net.sf.baseb.clients.agent.test.2");

		int id = test_create("net.sf.baseb.clients.agent.test.1", 8888, true);
		int id2 = test_create("net.sf.baseb.clients.agent.test.2", 8889, false);
		test_load(id);
		test_list(-1);
		
		// Extra test: create settings for plugin
		int pluginId = TestPluginDefinition.test_get("net.sf.basedb.plugins.HelpExporter");
		test_create_settings(id, pluginId);
		test_list_settings(id, 1);
		
		// Extra test: create job and list it for the agent
		int jobId = test_create_job(id, pluginId);
		test_list_jobs(id, 1);
		
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		TestJob.test_delete(jobId);
		test_delete_settings(id, pluginId);

		// Standard test: Delete
		test_delete(id2);
		test_delete(id);
		
		stop_fake_jobagent(8888);
		stop_fake_jobagent(8889);
		write("++Testing job agents "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static int test_create(String externalId, Integer port, boolean setAll)
	{
		if (!TestUtil.hasPermission(Permission.CREATE, Item.JOBAGENT)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			JobAgent j = JobAgent.getNew(dc, externalId);
			if (setAll)
			{
				j.setName("Test job agent");
				j.setDescription("Added at "+new Date());
			}
			if (port != null) 
			{
				j.setServer(InetAddress.getLocalHost().getHostAddress());
				j.setPort(port);
			}
			dc.saveItem(j);
			dc.commit();
			id = j.getId();
			write_item(0, j);
			write("--Create job agent OK");
		}
		catch (Throwable ex)
		{
			write("--Create job agent FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			JobAgent j = JobAgent.getById(dc, id);
			write_item(0, j);
			write("--Load job agent OK");
		}
		catch (Throwable ex)
		{
			write("--Load job agent FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void test_list(int expectedResults)
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemResultList<JobAgent> l = JobAgent.getQuery().list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List job agents OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List job agents FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_delete(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			JobAgent j = JobAgent.getById(dc, id);
			dc.deleteItem(j);
			dc.commit();
			write("--Delete job agent OK");
		}
		catch (Throwable ex)
		{
			write("--Delete job agent FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void write(String message)
	{
		System.out.println(message);
	}
	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tName      \tDescription\tExternal id\tServer\tPort\tCPU Usage\tMemory usage");
			write("-- \t-- \t--------- \t-----------\t-----------\t------\t----\t---------\t------------");
		}
	}
	static void write_item(int i, JobAgent j)
		throws BaseException
	{
		if (!TestUtil.getSilent())
		{
			System.out.println(i+":\t"+j.getId()+"\t"+j.getName()+"\t"+j.getDescription()+"\t"+
				j.getExternalId()+"\t"+j.getServer()+"\t"+j.getPort()+"\t"+
				j.getCpuUsage()+"%\t"+j.getUsedMemory());
		}
	}
	static void write_item(int i, JobAgentSettings j)
		throws BaseException
	{
		if (!TestUtil.getSilent())
		{
			System.out.println(i+":\t"+j.getId()+"\t"+j.getPluginDefinition()+"\t"+j.getJarPath()+"\t"+j.isTrusted()+"\t"+
					j.getMaxMemory()+"\t"+j.getPriorityBoost());
		}
	}
	static void write_item(int i, Job j, JobAgentSettings settings)
		throws BaseException
	{
		if (!TestUtil.getSilent())
		{
			System.out.println(i+":\t"+j.getId()+"\t"+j.getName()+"\t"+j.getDescription()+"\t"+j.getPluginDefinition()+"\t"+
					j.getPriority()+"\t"+settings.getPriorityBoost()+"\t"+settings.getEffectivePriority(j));
		}
	}
	
	static void test_create_settings(int agentId, int pluginId)
	{
		if (agentId == 0 || pluginId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			JobAgent j = JobAgent.getById(dc, agentId);
			PluginDefinition plugin = PluginDefinition.getById(dc, pluginId);
			JobAgentSettings settings = j.getSettings(plugin, true);
			settings.setPriorityBoost(1);
			settings.setTrusted(false);
			settings.setMaxMemory(50l*1024l*1024l);
			if (!settings.isInDatabase()) dc.saveItem(settings);
			dc.commit();
			dc = TestUtil.getDbControl();
			dc.reattachItem(settings);
			write_item(0, settings);
			write("--Create settings for job agent OK");
		}
		catch (Throwable ex)
		{
			write("--Create settings for job agent FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_list_settings(int agentId, int expectedResults)
	{
		if (agentId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			JobAgent j = JobAgent.getById(dc, agentId);
			ItemResultList<JobAgentSettings> l = j.getSettings().list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List settings for job agent OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List settings for job agent FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void test_delete_settings(int agentId, int pluginId)
	{
		if (agentId == 0 || pluginId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			JobAgent j = JobAgent.getById(dc, agentId);
			PluginDefinition plugin = PluginDefinition.getById(dc, pluginId);
			JobAgentSettings settings = j.getSettings(plugin, false);
			if (settings != null && settings.isInDatabase()) dc.deleteItem(settings);
			dc.commit();
			write("--Delete settings for job agent OK");
		}
		catch (Throwable ex)
		{
			write("--Delete settings for job agent FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static int test_create_job(int agentId, int pluginId)
	{
		if (pluginId == 0 || !TestUtil.hasPermission(Permission.CREATE, Item.JOB)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			PluginDefinition plugin = PluginDefinition.getById(dc, pluginId);
			Job job = Job.getNew(dc, plugin, null, null);
			job.setName(plugin.getName());
			job.setParameterValue("dummy", new StringParameterType(), "dummy"); // To end up with status = WAITING
			dc.saveItem(job);
			dc.commit();
			id = job.getId();
			dc = TestUtil.getDbControl();
			dc.reattachItem(job);
			write_item(0, job, JobAgent.getById(dc, agentId).getSettings(plugin, true));
			write("--Create job OK");
		}
		catch (Throwable ex)
		{
			write("--Create job FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		
		return id;		
	}
	
	static void test_list_jobs(int agentId, int expectedResults)
	{
		if (agentId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			JobAgent j = JobAgent.getById(dc, agentId);
			ItemResultList<Job> l = j.getWaitingJobs().list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				Job job = l.get(i);
				PluginDefinition plugin = job.getPluginDefinition();
				JobAgentSettings settings = j.getSettings(plugin, true);
				write_item(i, job, settings);
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List jobs for job agent OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List jobs for job agent FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void create_fake_jobagent(final int port, String externalId)
	{
		try
		{
			// The request handler for incoming requests
			Properties p = new Properties();
			p.setProperty("agent.port", Integer.toString(port));
			p.setProperty("agent.user", TestUtil.getLogin());
			p.setProperty("agent.password", TestUtil.getPassword());
			p.setProperty("agent.id", externalId);
			Agent agent = new Agent(p);
			RequestHandler requestHandler = new DefaultRequestHandler(agent)
			{
				public String handleCmd(Socket incoming, String cmd)
				{
					if (!TestUtil.getSilent()) write("--Job agent received cmd: " + cmd + " (port="+port+")");
					String answer = super.handleCmd(incoming, cmd);
					if (!TestUtil.getSilent()) write("--Job agent answers: " + answer);
					return answer;
				}
			};
			agent.service(requestHandler); // NOTE! Creates new thread for the listener
			write("--Create fake job agent OK (port " + port + ")");
		}
		catch (Throwable t)
		{
			write("--Create fake job agent FAILED (port " + port + ")");
			t.printStackTrace();
			ok = false;
		}
	}

	
	static void stop_fake_jobagent(int port)
	{
		JobAgentConnection conn = null;
		try
		{
			conn = new JobAgentConnection(port, 1000);
			conn.sendStop();
			write("--Stop fake job agent OK (port " + port + ")");
		}
		catch (Throwable t)
		{
			write("--Stop fake job agent FAILED (port " + port + ")");
			t.printStackTrace();
			ok = false;
		}
		finally
		{
			if (conn != null) conn.close();
		}
	}
}
