/*
 $Id$

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Johan Enell, Jari Hakkinen, Nicklas Nordborg
	Copyright (C) 2007 Nicklas Nordborg
 
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

 BASE is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 3
 of the License, or (at your option) any later version.

 BASE is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with BASE. If not, see <http://www.gnu.org/licenses/>.
 */
import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.File;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.Job;
import net.sf.basedb.core.ParameterType;
import net.sf.basedb.core.Permission;
import net.sf.basedb.core.PlateType;
import net.sf.basedb.core.PluginConfiguration;
import net.sf.basedb.core.PluginConfigurationRequest;
import net.sf.basedb.core.PluginDefinition;
import net.sf.basedb.core.PluginParameter;
import net.sf.basedb.core.PluginResponse;
import net.sf.basedb.core.RequestInformation;
import net.sf.basedb.core.plugin.Response;

public class TestPlateFlatFileImporter
{
	static boolean ok = true;

	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing plate import using plugin");
		int pluginDefinitionId  = TestPluginDefinition.test_get("net.sf.basedb.plugins.PlateFlatFileImporter");
		// Test 96 well plate
		// Create upload file and platetype
		TestReporter.test_import_from_file("data/test.plate96.import.txt", "Plate\\tRow\\tColumn\\tCluster ID\\tClone ID\\tGene Symbol\\tBarcode", "\\t", 4, 5);
		int fileId = TestFile.test_create("data/test.plate96.import.txt", false, false);
		int plateGeometry = TestPlateGeometry.test_create(8, 12, false);
		int plateTypeId = TestPlateType.test_create(plateGeometry, false);
		
		// Create plugin configuration and job
		int pluginConfigurationId = test_create_configuration96(pluginDefinitionId);
		TestPluginConfiguration.test_load(pluginConfigurationId);
		int jobId = test_create_job(pluginConfigurationId, fileId, plateTypeId);
		
		// Execute job
		ok &= TestJob.test_execute(jobId);
		
		// Test: list the result
		TestPlate.write_header();
		TestPlate.test_list(-1);
		
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();

		// Cleanup
		TestJob.test_delete(jobId);
		TestPluginConfiguration.test_delete(pluginConfigurationId);
		TestPlate.test_delete_all();
		TestPlateType.test_delete(plateTypeId);
		TestPlateGeometry.test_delete(plateGeometry);
		TestReporter.test_delete();
		TestFile.test_delete(fileId);

		write("++Testing plate import using plugin "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static int test_create_configuration96(int pluginDefinitionId)
	{
		if (pluginDefinitionId == 0 || !TestUtil.hasPermission(Permission.CREATE, Item.PLUGINCONFIGURATION)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();

			PluginDefinition pd = PluginDefinition.getById(dc, pluginDefinitionId);
			PluginConfiguration pc = pd.newPluginConfiguration();
			pc.setName(pd.getName());
			
			PluginConfigurationRequest request = pc.configure();

			write_request_information(request.getRequestInformation());
			
			request.setParameterValue("dataSplitterRegexp", "\\t");
			request.setParameterValue("dataHeaderRegexp", "Plate.*");
			
			request.setParameterValue("nameColumnMapping", "\\Plate\\");
			request.setParameterValue("rowColumnMapping", "\\Row\\");
			request.setParameterValue("columnColumnMapping", "\\Column\\");
			request.setParameterValue("reporterColumnMapping", "\\Clone ID\\");
			request.setParameterValue("barcodeColumnMapping", "\\Barcode\\");
			
			request.setParameterValue("minDataColumns", 7);
			request.setParameterValue("maxDataColumns", 7);
			
			PluginResponse response = request.invoke();
			if (response.getStatus() == Response.Status.DONE)
			{
				response.saveParameters(dc);
				dc.saveItem(pc);
				dc.commit();
			}
			else
			{
				throw new BaseException(response.getMessage(), response.getErrorList().get(0));
			}
			id = pc.getId();
			write("--Create configuration for 96 plate import OK");
		}
		catch (Throwable ex)
		{
			write("--Create configuration for 96 plate import FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static int test_create_job(int pluginConfigurationId, int fileId, int plateTypeId)
	{
		if (plateTypeId == 0 || pluginConfigurationId == 0 || fileId == 0 || !TestUtil.hasPermission(Permission.CREATE, Item.JOB)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();

			PluginConfiguration pc = PluginConfiguration.getById(dc, pluginConfigurationId);
			Job j = pc.newJob(null);
			j.setName(pc.getName());
			
			PluginConfigurationRequest request = j.configure(null);
			write_request_information(request.getRequestInformation());
			request.setParameterValue("file", File.getById(dc, fileId));
			request.setParameterValue("plateType", PlateType.getById(dc, plateTypeId));
			
			PluginResponse response = request.invoke();
			if (response.getStatus() == Response.Status.DONE)
			{
				response.saveParameters(dc);
				dc.saveItem(j);
				dc.commit();
			}
			else
			{
				throw new BaseException(response.getMessage(), response.getErrorList().get(0));
			}
			id = j.getId();
			write("--Create job for plate import OK");
		}
		catch (Throwable ex)
		{
			write("--Create job for plate import FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}
	
	static void write(String message)
	{
		System.out.println(message);
	}


	static void write_request_information(RequestInformation ri)
	{
		if (TestUtil.getSilent()) return;

		write("Request information for command: "+ri.getCommand());
		write("\tTitle:\t"+ri.getTitle());
		write("\tDescription:\t"+ri.getDescription());
		for (PluginParameter pp : ri.getParameters())
		{
			ParameterType pt = pp.getParameterType();
			System.out.println("\tParameter:\t"+pp.getName() + "\t" + pp.getLabel() + "\t" + pt );
		}
	}
}
