/*
	$Id: TestReporterType.java 4514 2008-09-11 20:07:12Z jari $

	Copyright (C) 2005 Samuel Andersson, Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import net.sf.basedb.core.*;
import java.util.Date;

public class TestReporterType
{

	static boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing reporter type");
		write_header();
		// Standard tests: create, load, list
		int id = test_create();
		test_load(id);
		test_list();

		// Standard test: Delete
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		test_delete(id);
		write("++Testing reporter type "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static int test_create()
	{
		if (!TestUtil.hasPermission(Permission.CREATE, Item.REPORTERTYPE)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ReporterType rt = ReporterType.getNew(dc);
			rt.setName("Test reporter type");
			rt.setDescription("Added at "+new Date());
			dc.saveItem(rt);
			dc.commit();
			id = rt.getId();
			write_item(0, rt);
			write("--Create reporter type OK");
		}
		catch (Throwable ex)
		{
			write("--Create reporter type FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ReporterType rt = ReporterType.getById(dc, id);
			write_item(0, rt);
			write("--Load reporter type OK");
		}
		catch (Throwable ex)
		{
			write("--Load reporter type FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_list()
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemResultList<ReporterType> l = ReporterType.getQuery().list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			write("--List reporter types OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List reporter types FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_delete(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ReporterType rt = ReporterType.getById(dc, id);
			dc.deleteItem(rt);
			dc.commit();
			write("--Delete reporter type OK");
		}
		catch (Throwable ex)
		{
			write("--Delete reporter type FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tName      \tDescription\tType");
			write("-- \t-- \t--------- \t-----------\t----");
		}
	}
	static void write_item(int i, ReporterType rt)
		throws BaseException
	{
		if (!TestUtil.getSilent()) System.out.println(i+":\t"+rt.getId()+"\t"+rt.getName()+"\t"+
			rt.getDescription());
	}
	static void write(String message)
	{
		System.out.println(message);
	}





}

