/**
	$Id: FilterTest.java 4514 2008-09-11 20:07:12Z jari $

	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.test.performance;


import net.sf.basedb.core.BioAssaySet;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Experiment;
import net.sf.basedb.core.ItemNotFoundException;
import net.sf.basedb.core.ItemParameterType;
import net.sf.basedb.core.Job;
import net.sf.basedb.core.PluginDefinition;
import net.sf.basedb.core.StringParameterType;
import net.sf.basedb.test.PluginUtil;
import net.sf.basedb.test.TestUtil;
import net.sf.basedb.util.Values;

/**
	Create a filtered bioassayset.

	@author nicklas
	@version 2.0
	@base.modified $Date: 2008-09-11 22:07:12 +0200 (Thu, 11 Sep 2008) $
*/
public class FilterTest
{

	public static boolean ok = true;

	/**
		Run the tests in this class.
	*/
	public static boolean run(String user, String password)
	{
		DbControl dc = null;
		TestUtil.write("++Entering filter test\n");
		try
		{
			TestUtil.login(user, password, false);

			// Create raw bioassays
			dc = TestUtil.getDbControl();
			
			// Load prepared items
			Experiment experiment = PrepareTest.getExperiment(dc);
			String expression = Config.getFilterExpression();
			BioAssaySet root = Util.findBioAssaySet(dc, experiment, "Root%");
			long numBioAssays = root.getBioAssays().count(dc);
			if (root == null)
			{
				throw new ItemNotFoundException("Root bioassayset");
			}
			
			long num = experiment.getBioAssaySets().count(dc) + 1;
			dc.close();

			long time = createFilteredBioAssaySet(root, "Filtered #" + num, expression) / 1000;
			
			TestUtil.write("---------------------------------------------\n");
			TestUtil.write("Source: " + root.getName() + "\n");
			TestUtil.write("  BioAssasys: " + numBioAssays + "\n");
			TestUtil.write("  Spots: " + root.getNumSpots() + "\n");
			TestUtil.write("  Reporters: " + root.getNumReporters() + "\n");
			TestUtil.write("Expression=" + expression + "\n");
			TestUtil.write("Create filtered bioassayset from " + root.getName() + ": " + 
					Values.formatTime(time) + "\n");
			TestUtil.write("---------------------------------------------\n");
		
			TestUtil.logout();
		}
		catch (Throwable t)
		{
			ok = false;
			t.printStackTrace();
		}
		finally
		{
			if (dc != null) dc.close();
		}
		TestUtil.write("++Leaving filter test (" + (ok ? "OK" : "FAILED") + ")\n");
		return ok;
	}

	/**
		Create a filtered bioassay set.
	*/
	public static long createFilteredBioAssaySet(BioAssaySet bioAssaySet, String name, String expression)
		throws Exception
	{
		TestUtil.write("--Create filtered bioassay set: " + name + "\n");
		DbControl dc = TestUtil.getDbControl();
		try
		{		
			PluginDefinition plugin = PluginDefinition.getByClassName(dc, "net.sf.basedb.plugins.FormulaFilter");
			Job job = Job.getNew(dc, plugin, null, bioAssaySet.getExperiment());
			job.setName("PerformanceTest: Create filtered bioassay set: " + name);
			job.setSendMessage(false);
			job.setRemoveJobWhenFinished(true);
			job.setParameterValue("source", 
					new ItemParameterType<BioAssaySet>(BioAssaySet.class, null), bioAssaySet);
			job.setParameterValue("childName", 
					new StringParameterType(255, null, true), name);
			job.setParameterValue("expression", 
					new StringParameterType(255, null, true), expression);
			dc.saveItem(job);
			dc.commit();
			long time = System.currentTimeMillis();
			PluginUtil.executeJob(job);
			return  System.currentTimeMillis() - time;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	
}
