/**
	$Id: ServiceFactory.java 4513 2008-09-11 20:06:36Z jari $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.ws.client;

import org.apache.axis2.AxisFault;
import org.apache.axis2.addressing.EndpointReference;
import org.apache.axis2.rpc.client.RPCServiceClient;

/**
	Interface that should be implemented by classes that can
	create {@link RPCServiceClient} objects. The service factory
	is used by {@link AbstractRPCClient#getService()} to create
	a {@link RPCServiceClient} object.
	<p>
	
	The service factory to use can be set on three levels:
	<ul>
	<li>Application-wide: {@link Factories#setDefaultServiceFactory(ServiceFactory)}

	<li>Per session: {@link 
		SessionClient#SessionClient(String, String, String, ServiceFactory)}
	
	<li>Per request: {@link AbstractRPCClient#setServiceFactory(ServiceFactory)}
	</ul>
	
	If no service factory has been specified the default application wide service
	is used, which is just a {@link ConfigurableServiceFactory} with all
	options at their default values.

	@author nicklas
	@version 2.8
	@base.modified $Date: 2008-09-11 22:06:36 +0200 (Thu, 11 Sep 2008) $
*/
public interface ServiceFactory
{
	
	/**
		Create a new {@link RPCServiceClient} object.
		@param target The URL to the web service
	*/
	public RPCServiceClient createService(EndpointReference target)
		throws AxisFault;

}
