<%-- $Id: edit_role.jsp 4587 2008-10-15 14:40:47Z martin $
	------------------------------------------------------------------
	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg, Martin Svensson
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------


	@author Nicklas
	@version 2.0
--%>
<%@ page session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.SystemItems"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.Include"
	import="net.sf.basedb.core.Permission"
	import="net.sf.basedb.core.User"
	import="net.sf.basedb.core.Role"
	import="net.sf.basedb.core.RoleKey"
	import="net.sf.basedb.core.ItemQuery"
	import="net.sf.basedb.core.ItemResultList"
	import="net.sf.basedb.core.BaseException"
	import="net.sf.basedb.core.PermissionDeniedException"
	import="net.sf.basedb.core.query.Orders"
	import="net.sf.basedb.core.query.Hql"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.PermissionUtil"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.util.Values"
	import="java.util.EnumSet"
	import="java.util.Set"
	import="java.util.Map"
	import="java.util.List"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%@ taglib prefix="t" uri="/WEB-INF/tab.tld" %>
<%!
private static final int DENY_CODE = PermissionUtil.getPermissionCode(EnumSet.of(Permission.DENIED));
private static final int CREATE_CODE = PermissionUtil.getPermissionCode(EnumSet.of(Permission.CREATE));
private static final int READ_CODE = PermissionUtil.getPermissionCode(EnumSet.of(Permission.READ));
private static final int USE_CODE = PermissionUtil.getPermissionCode(EnumSet.of(Permission.USE));
private static final int WRITE_CODE = PermissionUtil.getPermissionCode(EnumSet.of(Permission.WRITE));
private static final int DELETE_CODE = PermissionUtil.getPermissionCode(EnumSet.of(Permission.DELETE));
private static final int SET_OWNER_CODE = PermissionUtil.getPermissionCode(EnumSet.of(Permission.SET_OWNER));
private static final int SET_PERMISSION_CODE = PermissionUtil.getPermissionCode(EnumSet.of(Permission.SET_PERMISSION));

private String getOption(DbControl dc, Role role, Item itemType)
	throws BaseException
{
	int keyId = SystemItems.getRoleKeyId(itemType);
	if (keyId == 0) return "";

	RoleKey key = RoleKey.getById(dc, keyId);
	Set<Permission> permissions = role == null ? EnumSet.noneOf(Permission.class) : key.getPermissions(role);
	StringBuilder sb = new StringBuilder();
	sb.append("<option value=\"").append(key.getItemType().name()).append("\">");
	sb.append(HTML.encodeTags(key.getName()));
	sb.append(" [");
	if (permissions.contains(Permission.DENIED)) 
	{
		sb.append("DENIED");
	}
	else
	{
		Set<Permission> defined = itemType.getDefinedPermissions();
		if (defined != null)
		{
			if (defined.contains(Permission.CREATE)) sb.append(permissions.contains(Permission.CREATE) ? "C" : "-");
			if (defined.contains(Permission.READ)) sb.append(permissions.contains(Permission.READ) ? "R" : "-");
			if (defined.contains(Permission.USE)) sb.append(permissions.contains(Permission.USE) ? "U" : "-");
			if (defined.contains(Permission.WRITE)) sb.append(permissions.contains(Permission.WRITE) ? "W" : "-");
			if (defined.contains(Permission.DELETE)) sb.append(permissions.contains(Permission.DELETE) ? "D" : "-");
			if (defined.contains(Permission.SET_OWNER)) sb.append(permissions.contains(Permission.SET_OWNER) ? "O" : "-");
			if (defined.contains(Permission.SET_PERMISSION)) sb.append(permissions.contains(Permission.SET_PERMISSION) ? "P" : "-");
		}
	}
	sb.append("]");
	return sb.toString();
}
%>
<%
final Item itemType = Item.ROLE;
final SessionControl sc = Base.getExistingSessionControl(pageContext, true);
final ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, null, null);
final int itemId = cc.getId();
final String ID = sc.getId();
final float scale = Base.getScale(sc);
final DbControl dc = sc.newDbControl();
try
{
	String title = null;
	Role role = null;
	boolean isDefault = false;

	if (itemId == 0)
	{
		title = "Create role";
		cc.removeObject("item");
		isDefault = Values.getBoolean(cc.getPropertyValue("default"), false);
	}
	else
	{
		role = Role.getById(dc, itemId);
		cc.setObject("item", role);
		isDefault = role.isDefault();
		title = "Edit role -- " + HTML.encodeTags(role.getName());
		
	}
	if (role != null && !role.hasPermission(Permission.WRITE))
	{
		throw new PermissionDeniedException(Permission.WRITE, itemType.toString());
	}
	
	final boolean useUsers = sc.hasPermission(Permission.WRITE, Item.USER);
	RoleKey system = RoleKey.getById(dc, SystemItems.getRoleKeyId(Item.SYSTEM));
	final boolean hasShareToEveryone = role == null ? false : system.getPermissions(role).contains(Permission.SHARE_TO_EVERYONE);
	final boolean hasActAsAnotherUser = role == null ? false : system.getPermissions(role).contains(Permission.ACT_AS_ANOTHER_USER);
	final boolean hasSelectJobagent = role == null ? false : system.getPermissions(role).contains(Permission.SELECT_JOBAGENT);
	
	// Query to retrieve role keys
	final ItemQuery<RoleKey> roleKeyQuery = RoleKey.getQuery();
	roleKeyQuery.order(Orders.asc(Hql.property("name")));

	// Query to retrieve user members
	ItemQuery<User> userQuery = null;
	if (role != null)
	{
		userQuery = role.getUsers();
		userQuery.include(Include.ALL);
		userQuery.order(Orders.asc(Hql.property("name")));
	}

	final String clazz = "class=\"text\"";
	final String requiredClazz = "class=\"text required\"";
	%>

	<base:page type="popup" title="<%=title%>">
	<base:head scripts="tabcontrol.js,linkitems.js" styles="tabcontrol.css">
		<script language="JavaScript">
		// Validate the "Role" tab
		function validateRole()
		{
			var frm = document.forms['role'];
			if (Main.trimString(frm.name.value) == '')
			{
				alert("You must enter a name");
				frm.name.focus();
				return false;
			}
			return true;
		}
		// Validate the "Members" tab
		function validateMembers()
		{
			return true;
		}

		// Submit the form
		function saveSettings()
		{
			var frm = document.forms['role'];
			if (TabControl.validateActiveTab('settings'))
			{
				frm.addUsers.value = Link.getActionIds(1, 'U').join(',');
				frm.removeUsers.value = Link.getActionIds(-1, 'U').join(',');
				var systemPermission = 0;
				if (frm.share_to_everyone.checked) systemPermission += <%=PermissionUtil.getPermissionCode(EnumSet.of(Permission.SHARE_TO_EVERYONE))%>;
				if (frm.act_as_another_user.checked) systemPermission += <%=PermissionUtil.getPermissionCode(EnumSet.of(Permission.ACT_AS_ANOTHER_USER))%>;
				if (frm.select_jobagent.checked) systemPermission += <%=PermissionUtil.getPermissionCode(EnumSet.of(Permission.SELECT_JOBAGENT))%>;
				frm['<%=Item.SYSTEM.name()%>'].value = systemPermission;
				frm.submit();
			}
		}
		
		function init()
		{
			<%
			if (role == null)
			{
				%>
				var frm = document.forms['role'];
				frm.name.focus();
				frm.name.select();
				<%
			}
			%>
			initMembers();
		}
		function initMembers()
		{
			var members = document.forms['role'].members;
			<%
			if (userQuery != null)
			{
				ItemResultList<User> users = userQuery.list(dc);
				for (User user : users)
				{
					%>
					Link.addNewItem(members, new Item('U', <%=user.getId()%>, '<%=HTML.javaScriptEncode(user.getName())%>'));
					<%
				}
			}
			%>
		}
		function addUsersOnClick()
		{
			var users = Link.getListIds(document.forms['role'].members, 'U');
			var url = '../users/index.jsp?ID=<%=ID%>&cmd=UpdateContext&mode=selectmultiple&callback=addUserCallback';
			url += '&exclude='+users.join(',');
			Main.openPopup(url, 'AddUsers', 1000, 700);
		}
		function addUserCallback(userId, name)
		{
			var item = Link.getItem('U', userId);
			if (!item) item = new Item('U', userId, name);
			Link.addItem(document.forms['role'].members, item);
		}
		function removeOnClick()
		{
			Link.removeSelected(document.forms['role'].members);
		}
		function showPermissions(itemType)
		{
			var frm = document.forms['role'];
			if (itemType)
			{
				var permissionCode = frm[itemType].value;
				var definedPermissions = parseInt(frm[itemType+'_defined'].value);
				frm['deny'].checked = (permissionCode & <%=DENY_CODE%>);
				frm['create'].checked = (permissionCode & <%=CREATE_CODE%>);
				frm['create'].disabled = !(definedPermissions & <%=CREATE_CODE%>);
				frm['read'].checked = (permissionCode & <%=READ_CODE%>);
				frm['read'].disabled = !(definedPermissions & <%=READ_CODE%>);
				frm['use'].checked = (permissionCode & <%=USE_CODE%>);
				frm['use'].disabled = !(definedPermissions & <%=USE_CODE%>);
				frm['write'].checked = (permissionCode & <%=WRITE_CODE%>);
				frm['write'].disabled = !(definedPermissions & <%=WRITE_CODE%>);
				frm['delete'].checked = (permissionCode & <%=DELETE_CODE%>);
				frm['delete'].disabled = !(definedPermissions & <%=DELETE_CODE%>);
				frm['set_owner'].checked = (permissionCode & <%=SET_OWNER_CODE%>);
				frm['set_owner'].disabled = !(definedPermissions & <%=SET_OWNER_CODE%>);
				frm['set_permission'].checked = (permissionCode & <%=SET_PERMISSION_CODE%>);
				frm['set_permission'].disabled = !(definedPermissions & <%=SET_PERMISSION_CODE%>);
			}
		}
		function getPermissionCode()
		{
			var permissionCode = 0;
			var frm = document.forms['role'];
			permissionCode += frm['deny'].checked ? <%=DENY_CODE%>: 0;
			permissionCode += frm['create'].checked ? <%=CREATE_CODE%>: 0;
			permissionCode += frm['read'].checked ? <%=READ_CODE%>: 0;
			permissionCode += frm['use'].checked ? <%=USE_CODE%>: 0;
			permissionCode += frm['write'].checked ? <%=WRITE_CODE%>: 0;
			permissionCode += frm['delete'].checked ? <%=DELETE_CODE%>: 0;
			permissionCode += frm['set_owner'].checked ? <%=SET_OWNER_CODE%>: 0;
			permissionCode += frm['set_permission'].checked ? <%=SET_PERMISSION_CODE%>: 0;
			return permissionCode;
		}
		
		function correctPermissions(checked, forceDeny)
		{
			var frm = document.forms['role'];
			if (checked)
			{
				frm['deny'].checked = frm['deny'].checked && forceDeny;
				frm['set_permission'].checked = frm['set_permission'].checked && !frm['deny'].checked;
				frm['set_owner'].checked = frm['set_owner'].checked && !frm['deny'].checked;
				frm['delete'].checked = frm['delete'].checked && !frm['deny'].checked;
				frm['write'].checked = (frm['write'].checked || frm['delete'].checked || frm['set_owner'].checked || frm['set_permission'].checked) && !frm['deny'].checked;
				frm['use'].checked = (frm['use'].checked || frm['write'].checked) && !frm['deny'].checked;
				frm['read'].checked = (frm['read'].checked || frm['use'].checked) && !frm['deny'].checked;
				frm['create'].checked = frm['create'].checked && !frm['deny'].checked;
			}
			else
			{
				frm['deny'].checked = frm['deny'].checked && !frm['create'].checked && !frm['read'].checked && !frm['use'].checked && !frm['write'].checked && !frm['delete'].checked && !frm['set_owner'].checked && !frm['set_permission'].checked;
				frm['create'].checked = frm['create'].checked && !frm['deny'].checked;
				frm['read'].checked = frm['read'].checked && !frm['deny'].checked;
				frm['use'].checked = (frm['use'].checked && frm['read'].checked) && !frm['deny'].checked;
				frm['write'].checked = (frm['write'].checked && frm['use'].checked) && !frm['deny'].checked;
				frm['delete'].checked = (frm['delete'].checked && frm['write'].checked) && !frm['deny'].checked;
				frm['set_owner'].checked = (frm['set_owner'].checked && frm['write'].checked) && !frm['deny'].checked;
				frm['set_permission'].checked = (frm['set_permission'].checked && frm['write'].checked) && !frm['deny'].checked;
			}
		}

		function setPermissions(checked, forceDeny)
		{
			var frm = document.forms['role'];
			correctPermissions(checked, forceDeny);
			var permissionCode = getPermissionCode();

			for (var i = 0; i < frm.items.length; i++)	// >
			{
				var item = frm.items[i];
				var itemType = item.value;
				if (item.selected && itemType)
				{
					frm[itemType].value = permissionCode;
					var definedPermissions = parseInt(frm[itemType+'_defined'].value);
					var status = '';
					if (frm['deny'].checked)
					{
						status = "DENIED";
					}
					else
					{
						if (definedPermissions & <%=CREATE_CODE%>) status += frm['create'].checked ? "C" : "-";
						if (definedPermissions & <%=READ_CODE%>) status += frm['read'].checked ? "R" : "-";
						if (definedPermissions & <%=USE_CODE%>) status += frm['use'].checked ? "U" : "-";
						if (definedPermissions & <%=WRITE_CODE%>) status += frm['write'].checked ? "W" : "-";
						if (definedPermissions & <%=DELETE_CODE%>) status += frm['delete'].checked ? "D" : "-";
						if (definedPermissions & <%=SET_OWNER_CODE%>) status += frm['set_owner'].checked ? "O" : "-";
						if (definedPermissions & <%=SET_PERMISSION_CODE%>) status += frm['set_permission'].checked ? "P" : "-";
					}
					var text = item.text.replace(/\[.*\]/, '['+status+']');
					item.text = text; 
				}
			}
		}

		</script>
	</base:head>
	<base:body onload="init()">
		<p>
		<form action="index.jsp?ID=<%=ID%>" method="post" name="role" onsubmit="return false;">
		<input type="hidden" name="cmd" value="UpdateItem">

		<h3 class="docked"><%=title%> <base:help tabcontrol="settings" /></h3>
		<t:tabcontrol id="settings" contentstyle="<%="height: "+(int)(scale*300)+"px;"%>" 
			position="bottom" remember="<%=role != null%>">
		<t:tab id="info" title="Role" validate="validateRole()" helpid="role.edit">
			<table class="form" cellspacing=0>
			<tr>
				<td class="prompt">Name</td>
				<td><input <%=requiredClazz%> type="text" name="name" 
					value="<%=HTML.encodeTags(role == null ? Values.getString(cc.getPropertyValue("name"), "New role") : role.getName())%>" 
					size="40" maxlength="<%=Role.MAX_NAME_LENGTH%>"></td>
			</tr>
			<tr valign="top">
				<td class="prompt">System-level<br>permissions</td>
				<td>
					<input type="checkbox" value="1" name="share_to_everyone" 
						<%=hasShareToEveryone ? "checked" : ""%>
						>Share to Everyone<br>
					<input type="checkbox" value="1" name="act_as_another_user" 
						<%=hasActAsAnotherUser ? "checked" : ""%>
						>Act as another user<br>
					<input type="checkbox" value="1" name="select_jobagent" 
						<%=hasSelectJobagent ? "checked" : ""%>
						>Select job agent for jobs<br>
				</td>
			</tr>
			<tr>
				<td class="prompt">Default</td>
				<td>
					<input type="radio" name="is_default" value="0" <%=!isDefault ? "checked" : ""%>>no
					<input type="radio" name="is_default" value="1" <%=isDefault ? "checked" : ""%>>yes
				</td>
			</tr>	

			<tr valign=top>
				<td class="prompt">Description</td>
				<td nowrap>
					<textarea <%=clazz%> rows="4" cols="40" name="description" wrap="virtual"
						><%=HTML.encodeTags(role == null ? cc.getPropertyValue("description") : role.getDescription())%></textarea>
					<a href="javascript:Main.zoom('Description', 'role', 'description')"
						title="Edit in larger window"><base:icon image="zoom.gif" /></a>
				</td>
			</tr>
			</table>
			<div align=right>&nbsp;<i><base:icon image="required.gif" /> = required information</i></div>
		</t:tab>
		
		<t:tab id="permissions" title="Permissions" 
			tooltip="Set permissions for this role" helpid="role.edit.permissions">
		<table >
		<tr valign="top">
		<td>
			<b>Item types</b><br>
			<select name="items" size="15" onChange="showPermissions(this[this.selectedIndex].value);" multiple>
			<%
			Map<String, List<Item>> permissionGroups = PermissionUtil.getPermissionGroups();
			for (Map.Entry<String, List<Item>> entry : permissionGroups.entrySet())
			{
				String name = entry.getKey();
				List<Item> items = entry.getValue();
				%>
				<option class="selectoptionheader" value="">-- <%=name%>
				<%
				for (Item item : items)
				{
					%>
					<%=getOption(dc, role, item)%>
					<%
				}
			}
			%>
			</select>
		</td>
		<td>
			<b>Permissions</b><br>
			<input type="checkbox" name="deny" onClick="setPermissions(this.checked, true)">Deny<br>
			<input type="checkbox" name="create" onClick="setPermissions(this.checked)">Create<br>
			<input type="checkbox" name="read" onClick="setPermissions(this.checked)">Read<br>
			<input type="checkbox" name="use" onClick="setPermissions(this.checked)">Use<br>
			<input type="checkbox" name="write" onClick="setPermissions(this.checked)">Write<br>
			<input type="checkbox" name="delete" onClick="setPermissions(this.checked)">Delete<br>
			<input type="checkbox" name="set_owner" onClick="setPermissions(this.checked)">Set owner<br>
			<input type="checkbox" name="set_permission" onClick="setPermissions(this.checked)">Set permission<br>
			</td>
		</tr>
		</table>
		</t:tab>
		
		<t:tab id="members" title="Members" tooltip="Add/remove members of this role" 
			validate="validateMembers()" helpid="role.edit.members">
		<table >
		<tr valign="top">
		<td>
			<b>Members</b><br>
			<select name="members" size="15" multiple 
				<%=!useUsers ? "disabled readonly class=\"disabled\"" : ""%> style="width: 15em;">
			</select>
			<input type="hidden" name="removeUsers" value="">
			<input type="hidden" name="addUsers" value="">
		</td>
		<td>
			<br>
			<table width="150">
			<tr><td><base:button 
				onclick="addUsersOnClick()" 
				title="Add&nbsp;users&hellip;" 
				tooltip="Add users to this role"
				disabled="<%=!useUsers %>" 
				/></td></tr>
			<tr><td><base:button 
				onclick="removeOnClick()" 
				title="Remove" 
				tooltip="Remove the selected items from this role"
				disabled="<%=!useUsers%>" 
			/></td></tr>
			</table>
		</td>
		</tr>
		</table>
		</t:tab>
		</t:tabcontrol>

		<table align="center">
		<tr>
			<td width="50%"><base:button onclick="saveSettings()" title="Save" /></td>
			<td width="50%"><base:button onclick="window.close()" title="Cancel" /></td>
		</tr>
		</table>
		<%
		ItemResultList<RoleKey> roleKeys = roleKeyQuery.list(dc);
		for (RoleKey key : roleKeys)
		{
			int permissionCode = role == null ? 0 : PermissionUtil.getPermissionCode(key.getPermissions(role));
			int definedCode = key.getItemType().getDefinedPermissions() == null ? 0 : PermissionUtil.getPermissionCode(key.getItemType().getDefinedPermissions());
			String name = key.getItemType().name();
			%>
			<input type="hidden" name="<%=name%>" value="<%=permissionCode%>">
			<input type="hidden" name="<%=name%>_defined" value="<%=definedCode%>">
			<%
		}
		%>
		</form>
	</base:body>
	</base:page>
	<%
}
finally
{
	if (dc != null) dc.close();
}
%>
